/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */

/**************************************************************************
 * File: carmel.h       Driver Declaraitons for carmel.c. 
 **************************************************************************/

#ifndef _carmel_h
#define _carmel_h

#ifndef LINUX_VERSION_CODE
#include <linux/version.h>
#endif

#include  "constant.h"
#include  "type.h"

#ifndef KERNEL_VERSION
#define KERNEL_VERSION(x,y,z) (((x)<<16)+((y)<<8)+(z))
#endif

#ifndef MIN
#define MIN(a,b) (a<b?a:b)
#endif

#ifndef UTYPE
#define UTYPE
#define u8 unsigned char
#define U8 unsigned char
#define unchar unsigned char
#define UCHAR unsigned char
#define U16 unsigned short
#define ushort unsigned short
#define USHORT unsigned short
#define PUSHORT unsigned short *
#define U32 unsigned int
#define u32 unsigned int
#define PU32 unsigned int *
#define ULONG unsigned long
#define PVOID void *
#endif

#define driver_version_major 1
#define driver_version_minor 1
#define driver_version_OEMCode 0
#define driver_version_BuildNumber 18
#define	driver_version_BuildMinNum  ""

#define PENDING_IOCTL 		0x00000001
#define PENDING_FLUSH 		0x00000002
#define PENDING_INQUIRE 	0x00000004
#define PENDING_SYNC_TIME 	0x00000008
#define PENDING_ALLOC_BUFFER 	0x00000010
#define PENDING_GET_FW_VER 	0x00000020
#define PENDING_SCAN_CHANNEL 	0x00000040
#define PENDING_GET_CHANNEL 	0x00000080
#define	PENDING_READ_VERIFY	0x00000100
#define	PENDING_SET_TNCQ	0x00000200
#define	PENDING_RESET_ARRAY	0x00000400
#define	PENDING_RESET_CARMEL	0x00000800

#define PENDING_RES_SHORT  	0x00010000

#define MESSAGE_QUEUE_LENGTH 48
#define COMMAND_QUEUE_LENGTH 48

typedef enum {
    OS_MSG_RW		= 1,
    OS_MSG_GET_CAPACITY,
    OS_MSG_FLUSH,
    OS_MSG_IOCTL,
    OS_MSG_ARRAY,
    OS_MSG_CHANNEL_GET_CONTROL,
    OS_MSG_CHANNEL_RELEASE_CONTROL,
    OS_MSG_SET_TIME,
    OS_MSG_GET_FW_VER,
    OS_MSG_ALLOC_BUFFER,
    OS_MSG_DISK,
    OS_MSG_SCAN_CHANNEL,
    OS_MSG_READ_VERIFY,
    OS_MSG_PASSTHROUGH,
    OS_MSG_SETTNCQ,
    OS_MSG_RESETARRAY,
    OS_MSG_RESETCARMEL
} OS_CMD_TYPE;

/* INQUIRY data format */
typedef struct 
{
  unchar  type_qual;
  unchar  modif_rmb;
  unchar  version;
  unchar  resp_aenc;
  unchar  add_length;
  unchar  reserved1;
  unchar  reserved2;
  unchar  misc;
  unchar  vendor[8];
  unchar  product[16];
  unchar  revision[4];
} carmel_inq_data;

typedef struct
{
  carmel_inq_data gen_inq_data;
  unchar  enc_uniq_id[8];	/* zero */
  unchar  safteid[6];		/* "SAF-TE" */
  unchar  safte_rev[4]; 	/* 1.00 */
  unchar  vendor_unq_parm[42];  /* zero */
} carmel_safte_inq_data;

/* READ_CAPACITY data format */
typedef struct 
{
  u32  last_block_no;
  u32  block_length;
} carmel_rdcap_data;


#define	MAX_SECTOR_SIZE	256
#define	MAX_CAN_QUEUE	32
#define	MAX_SG_TABLESIZE 32

#ifdef  NULL
#undef  NULL
#endif
#define NULL ((void *) 0)
 
#define DEBUG (FALSE)

#if !defined(MAX_ADAPTORS)
#define MAX_ADAPTORS	 4
#endif

#define MAX_TARGETS	(16)
#define MAX_DRIVES       (8)
#define SECTOR_SIZE      (0x200) 
// 512 byte sectors

union msg_union
{
	char max[288];
	struct 
	{
		U8 type;
		U8 ArrayID;
		U8 SGCount;
		U8 SGType;
		U32 Handle;
		U32 LBA;
		U16 BlockCount;
		U16 LBAHigh;
		struct
		{
			U32 SGStart;
			U32 SGSize;
		} SG[32];
	} rw;
	struct
	{
		U8 type;
		U8 ArrayID;
		U8 reserved1;
		U8 reserved2;
		U32 Handle;
		U32 LBA;
		U16 BlockCount;
		U16 LBAHigh;
	} verify;
	struct
	{
		U8 type;
		U8 ArrayID;
		U16 reserved1;
		U32 Handle;
		U32 datapointer;
		U32 reserved2;
		U32 reserved3;
	} get_capacity;
	struct
	{
		U8 type;
		U8 notused[3];
		U32 Handle;
	} flush;
	struct
	{
		U8 type;
		U8 subtype;
		U8 ArrayID;
		U8 byte3;
		U32 Handle;
		U32 datapointer;
		U8 extrabyte[52];
	} ioctl;
	struct
	{
		U8 type;
		U8 subtype;
		U8 reserved[2];
		U32 Handle;
		U8 channel_count;
		U8 channelID[MAX_DRIVES];
	} ch_control;
	struct
	{
		U8 type;
		U8 subtype;
		U16 sReserved;
		U32 Handle;
		U32 lReserved;
		U32 TimeStamp;
	} sync_time;
	struct
	{
		U8 type;	/* should be 0x06 */
		U8 subtype;	/* should be 0x13 */
		U8 chanid;
		U8 reserved1;
		U32 handle;
		U32 reserved2;
		U32 qdepth;	/* 1 disable queue */
	} setntcq;
	struct
	{
		U8 type;
		U8 subtype;
		U8 arrayid;
		U8 reserved;
		U32 handle;
	} rstarray;
	struct
	{
		U8 type;
		U8 subtype;
		U8 SGCount;
		U8 SGType;
		U32 Handle;
		/*U32 lReserved;*/
		U32 StartAddress;
		U32 length;
		U32 event_log_pool;
		U32 event_log_count;
		U32 response_message_pool;
		U32 response_message_count;
		U32 command_message_pool;
		U32 command_message_count;
		struct
		{
			U32 SGStart;
			U32 SGSize;
		} SG[8];
	} alloc_buf;
	struct  {
		U8 type;
		U8 channel;
		U8 bReserved1;
		U8 bReserved2;
		U32 Handle;
		U32 datapointer;
		U16 databuffersize;
		U16 wReserved3;
		U32 commanddataaddr;
	} pass_thru;
};

typedef struct msglink{
	Scsi_Cmnd *cmnd;
	struct msglink *next;
	union msg_union *msg;
	void (*fn)(Scsi_Cmnd *);
	U16	type;
	U16	parameter;

	U16	parm1;	/* smbus id */
	U16	parm2;  /* flags: idel, verifyend, verifystart */
#define	VERIFY_IDEL	0
#define	VERIFY_END	1
#define	VERIFY_START	2

	U16	parm3;
	U16	parm4;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,4,12))

#define DECLARE_PCI_UNMAP_ADDR(ADDR_NAME)
#define DECLARE_PCI_UNMAP_LEN(LEN_NAME)
#define pci_unmap_addr(PTR, ADDR_NAME)		(0)
#define pci_unmap_addr_set(PTR, ADDR_NAME, VAL)	do { } while (0)
#define pci_unmap_len(PTR, LEN_NAME)		(0)
#define pci_unmap_len_set(PTR, LEN_NAME, VAL)	do { } while (0)

#elif (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))

	DECLARE_PCI_UNMAP_ADDR(mapping) 
	DECLARE_PCI_UNMAP_LEN(len)

#endif
} MSGLINK, *PMSGLINK;

typedef struct cc{
	Scsi_Cmnd	*OriginalSrb;
	Scsi_Cmnd	*cmd;
	void (*fn)(Scsi_Cmnd *);
	char c[16];
	u32 *sg;
	u16 *cc_buffer;
	struct scatterlist *current_sg;
	u32 sg_count;
	
	s32 b_count;
	u32 ModeSenseRetryCount;
	s32 words_todo;

	u8 RDP;
	u8 sa_same;
} CC, *PCC;

typedef	struct carmel_host 
{ 
  spinlock_t               spin_lock;
  spinlock_t               simple_lock;

  unsigned long flags;

#define	DRV_INITIALIZE	0x01
#define	DRV_DOWN	0x02
#define	DRV_IN_INTR	0x04
#define	DRV_IN_RESET	0x08

  int                      scsi_id;          /* host adapter SCSI ID */
  
  struct pci_dev          *pdev;
  
  dma_addr_t dma_handle;
  struct Scsi_Host        *host;             /* pointer to scsi host */
  int                      host_no;          /* SCSI host number */
  int TotalDiskCount;
  U32 HDCapacity[MAX_DRIVES];

  /*
   * in RAID_FEATURE RAID mode, it save SP0 register value
   * in RAID_FEATURE_BYPASS mode, its value present how 
   * many messages have been sent to each channel 
   */
  volatile union {
	unsigned int sp0;
	unsigned char chanworks[MAX_DRIVES];
  } chanactive;
  
  volatile u32	count;
  volatile u32	pending;

  /* For ATAPI Device */

  CC	cc[8];

  u8 bSupportDMACommand[8];
  u8 bSupportPIOCommand[8];
  u8 DiscsPresent[8];
  IDENTIFY_DATA IdentifyData[8];
  unsigned short DeviceDesc[MAX_DRIVES];
  unsigned short atapi_type[MAX_DRIVES];
  unsigned char channel_type[MAX_DRIVES];
  unsigned int DevicePresent;
  unsigned int TapeChannel;

  unsigned int major;            /* control device's major */
  int          counter;          /*counter of using control device*/

  wait_queue_head_t WaitQ;

  struct timer_list atapitm;
  struct timer_list flushtm;
  struct timer_list ledtm;

  volatile unsigned char channel_control;
  volatile unsigned char channel_number;
  volatile unsigned char raid_feature;
  volatile unsigned char init_fail;

  unsigned int	ioctl_status;

  unsigned int		irq;

  unsigned int product_type;
  unsigned int firmwareversion;

  unsigned char	bus_num;
  int	id;			/* i.e adapter no. */
  int   bTapeDMA;		/* default is off */
  int	present;

  CARMEL_WORKING_MODE mode;

  PMSGLINK	msg_head;
  PMSGLINK	msg_tail;

#if defined(CONFIG_X86_64)
  u64 mu_hi32_addr;
#endif
  unsigned char *r_msg_buffer;
  unsigned char *event_log_pool;
  int event_index_start;
  int event_index_end;
  unsigned char *free_buf;
  unsigned char *free_buf2;
  unsigned char *free_buf3;
  u8 *engine_buffer[8];
  unsigned char m_size[8];
  int	ResponseMessageIndex;
  unsigned char	*msLBA;
  unsigned char *flash;
  MSGLINK  msg_link[COMMAND_QUEUE_LENGTH];
#if !defined(CANREADMUREGISTER)
  int MUDMAcnt;		/* counter for how many mu dma doing */
#endif
  int msg_wait_queue_len;
  union msg_union *msg_wait_queue[COMMAND_QUEUE_LENGTH];

  HdAryInfo_t hdinfo[MAX_DRIVES];
  unsigned char SMARTFlag[MAX_DRIVES];
  /* Save PCI configure space for restoring it after carmel reset */
  unsigned int pci_conf_space[32];
} carmel_host_t;

struct carmel_host *carmel_hostp[MAX_ADAPTORS];

#endif /* _carmel_h */

