/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */


/*
 * Description   : Define IOCTLs and parameter for these IOCTLs, and other common data
 *                 structures between driver and API. 
 *                 This file can be included in both DRIVER source
 *                 code and RAIDAPI source code.
 *                 Controller info. is also defined in this file, for driver provides 
 *                 this structure.
 */


#ifndef _RAID_API_IOCTL_H_
#define _RAID_API_IOCTL_H_

#pragma pack(4)

/* Switch for SCSI pass through IOCTL */
/*#define IOCTL_SCSI_PASSTHROUGH */

#ifdef IOCTL_SCSI_PASSTHROUGH
typedef unsigned long ULONG_PTR;
#include <ntddscsi.h>
#endif


/*
 * About 4 channels Carmel and 8 channels Carmel:
 *   When use array(ShuZu) variables, just use this constant number 
 * as array elements length, but only frontal 4 elements are valid 
 * for 4 channels Carmel. 
 *   And when refer to given adapter's channels number, 
 * use CCard::GetNumberOfChannels(), Don't use these macros .
 */

#define MAX_ADAPTER_SUPPORT        4  /* Bypass mode 4, RAID mode 2 */
#define MAX_DISKID_PER_CHANNEL     2

#define MAX_CHANNEL_PER_ADAPTER    8  /* To support 4/8 channels, use this number for array variables. */
                                      /* But only frontal 4 elements are valid for 4 channels Carmel. */
#define MAX_DISKID_PER_ADAPTER     (MAX_CHANNEL_PER_ADAPTER*MAX_DISKID_PER_CHANNEL)
#define MAX_DISK_PER_ADAPTER       MAX_CHANNEL_PER_ADAPTER
#define MAX_ARRAY_PER_ADAPTER      MAX_DISK_PER_ADAPTER
#define MAX_DISK_PER_ARRAY         MAX_DISK_PER_ADAPTER 


#define IDENTIFY_DATA_LENGTH       512
#define MAX_ARY_NAME_LENGTH        40
#define MAX_DISK_NAME_LENGTH       40

/* fileds in Identify data */
#define MAX_MODULE_LENGTH          40
#define MAX_SERIAL_LENGTH          20
#define MAX_FWVERSION_LENGTH       8

#define EVENT_SIZE                 12  /* BYTE. If EVENTLOG structure is changed, this value needs to be verified again. */
#define MAX_EVENTS_BUF             20*EVENT_SIZE  /* Max event buffer size got from driver */


/* For SAF_TE Operation */
#define MAX_SAF_TE_BUFFER_SIZE     64

/* For TCQ/NCQ payload */
#define MAX_QDEPTH                 32
#define MIN_QDEPTH                 1


/*
 * user level controller info. which will be shown in the controller pane.
 * Driver provides this structure
 */
typedef struct 
{
	ULONG	ChannelNumber;	/* number of channels this controller have. */
	                        /* Need to check MAX_CHANNEL number, because it can be 4 ot 8 */
	BYTE	bName[32];	/* Controller name */

	ULONG	IRQ;		/* Interrupt number; */
	ULONG	SlotNumber;	/* Slot this controller take place; */
	ULONG	BIOSMemStart;	/* BIOS rom start address */
	ULONG   BIOSMemRange;	/* size of BIOS rom */

	ULONG   Mode;           /* controller mode: bridge or raid mode */

	/* In different mode , the following BARs are different  */
	ULONG pBAR[6];         /* BAR#0~BAR#5*/

	/*
	 * Newly added, FW version.
	 * Driver can bypass these 2 fields.
	 */
	ULONG  Version;
	ULONG  ProductType;     /* Use lower byte
		                 * Bit2:1 for 4-port device; 0 for 8-port device
				 * Bit1:Raid Feature,0 for RAID mode (support 0, 1, 10, JBOD); 1 for Pass-through
				 * Bit0:Zero Channel Raid present or not. 0 for not present; 1 for present. This bit
	                         *      is used only when adapter integrates Intel RAID chipset. And don't care it here. */
} CONTROLLERINFO, *PCONTROLLERINFO;


/* used when fetch ATAPI device information. */
enum enumTransferMode
{
	ModeUnknown=-1,
	DMA0,
	DMA1,
	DMA2,
	UDMA0,
	UDMA1,
	UDMA2,
	UDMA3,
	UDMA4,
	UDMA5,
	UDMA6,
	PIO3,
	PIO4
};



/*
 * For SCSI Pass Through IOCTL
 */
#ifdef IOCTL_SCSI_PASSTHROUGH
#define SPT_SENSE_LENGTH 32

/* Need to define: */
#define SCSI_PASSTHROUGH_CDB0	    0xE3
#define SCSI_PASSTHROUGH_TARGETID   7
#define SCSI_PASSTHROUGH_PATHID     0
#define SCSI_PASSTHROUGH_CDBLENGTH  12
#define SCSI_PASSTHROUGH_LUN        0
#define SCSI_PASSTHROUGH_TIMEOUT    10000

typedef struct _SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER
{
	SCSI_PASS_THROUGH_DIRECT sptd;
	ULONG             Filler;   /* realign buffer to double word boundary */
	UCHAR             ucSenseBuf[SPT_SENSE_LENGTH];
} SCSI_PASS_THROUGH_DIRECT_WITH_BUFFER, 
*PSCSI_PASS_THROUGH_DIRECT_WITH_BUFFER;
#endif


/* Driver had defined this structure in its header file */
typedef struct _SRB_IO_CONTROL {
	ULONG HeaderLength;
	UCHAR Signature[8];
	ULONG TimeOut;
	ULONG ControlCode;
	ULONG ReturnCode;
	ULONG Length;
} SRB_IO_CONTROL, *PSRB_IO_CONTROL;


/*SRB_IO_CONTROL. ReturnCode Indicates whether current opetation successful.*/
/*SRB_IO_CONTROL. ReturnCode: 0x00000000        SUCCESSFUL*/
/*SRB_IO_CONTROL. ReturnCode: 0xFFFFFFFF        FAILED*/
/*SRB_IO_CONTROL. ReturnCode: 0x00000001        BUSY   (Firmware busy now)*/

#define IOCONTROL_SUCCESSFUL      0x00000000
#define IOCONTROL_UNKNOWN         0xFFFFFFFF
#define IOCONTROL_BUSY            0x00000001
#define IOCONTROL_TYPE_INVALID    0x00000002    /* returncode == REQ_MSG_INVALID_TYPE */
#define IOCONTROL_SUBCMD_INVALID  0x00000003    /* returncode == REQ_MSG_INVALID_SUBCMD */
#define IOCONTROL_INVALID_DATA    0x00000004    /* returncode == REQ_MSG_INVALID_DATA */

/* Return code in Pass Through mode */
/* cxm added in RAID_API_IOCTL.h */
#define IOCONTROL_PASSTHRU_NA_CMD  0x00000005
#define IOCONTROL_PASSTHRU_ERROR   0x00000006


/*
 * Header for IO data inputted to driver to indicate whether
 * the input data is MessagePayload style. If it is not MessagePayload
 * style then Driver needs to handle it especially, otherwise just transfer
 * the input data to Firmware which processes the MessagePayload data.
 */
typedef struct {
	BYTE   SMARTDataBufFlag;/* Flag of SMART IOCTL frame data. */
	                        /*  bit0: 1 for SMART Operation has IN  DATA buffer, 0 for not. */
                                /*  bit1: 1 for SMART Operation has OUT DATA buffer, 0 for not. */
	BYTE   Reserved[2];
	/*ULONG  ulPayloadStyle;*/ /* Whether the IO data is MessagePayload style */
	BYTE   ulPayloadStyle;  /* Whether the IO data is MessagePayload style*/
	USHORT usINSize;        /* Size of MessagePalyload input parameter */
	                        /* Or if SAF_TE IOCTL, this indicates the IN buffer size */
	USHORT usOUTSize;       /* Size of MessagePalyload output parameter */
	                        /* Or if SAF_TE IOCTL, this indicates the OUT buffer size */
} IODATA_HEADER, * PIODATA_HEADER;

#define SMART_DATA_IN     0x01
#define SMART_DATA_OUT    0x02

/*
 * Signature in SRB_IO_CONTROL
 */
#define IOCTL_SIGNATURE       "CARMEL"


/*
 *  Layout of IO data inputted to driver:
 *   ______________________
 *   |  SRB_IO_CONTROL     |
 *   |_____________________|
 *   |  IODATA_HEADER      |
 *   |_____________________| 
 *   |                     |
 *   |     .....           |
 *   |     IODATA          |        
 *   |     .....           |
 *   |_____________________|
 *


 *
 *
 *  IOCTLs
 *

 *
 * Driver process
 */

#define IOCTL_GETVERSION	0x00000001
#define IOCTL_GETHARDWARETYPE	0x00000002
#define IOCTL_ADAPTERCOUNT	0x00000003
#define IOCTL_ADAPTER		0x00000004
#define IOCTL_ATAPIDEVICEINFO	0x00000005

/* EVENT Operation */
#define IOCTL_GETLOGCOUNT	0x00000008
#define IOCTL_GETLOGEVENT	0x00000009

/* ROM Operation */
#define IOCTL_READ_ROM_INFO	0x0000000A
#define IOCTL_READ_ROM_DATA	0x0000000B
#define IOCTL_WRITE_ROM_DATA	0x0000000C
#define IOCTL_ERASE_ROM		0x0000000E
#define	IOCTL_FWVECTOR		0x00000010

/* SMART Operation */
#define IOCTL_SMART_OPERATION      0x0000000D

/* Enclosure Operation */
#define IOCTL_ENCLOSURE_OPERATION  0x0000000F



/* Payload Style IOCTL */
#define IOCTL_PAYLOADSTYLE         0x20000000

/*
 * IOCTLs' Parameter
 */

typedef struct _PLACEHOLDER{
	BYTE PlaceHolder; 
}PLACEHOLDER;




/* Driver handle it */
typedef struct _GETVERSION_FRAME_OUT{
	BYTE Major;
	BYTE Minor;
	BYTE OEMCode;
	BYTE BuildNumber;
}GETVERSION_FRAME_OUT;
typedef union {
	PLACEHOLDER  _IN; /* This is just place holder.        */
			  /* For the sake of coding convenience*/
	GETVERSION_FRAME_OUT _OUT;
} IOCTL_GETVERSION_FRAME, *PIOCTL_GETVERSION_FRAME;

typedef struct _IOC_PKG_GETVERSION {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_GETVERSION_FRAME getv;
} IOC_PKG_GETVERSION, *PIOC_PKG_GETVERSION;

/* Driver handle it */
typedef struct _GETHARDWARETYPE_FRAME_OUT {
	ULONG ulType;
} GETHARDWARETYPE_FRAME_OUT;
typedef union {
	PLACEHOLDER _IN; /* This is just place holder.         */
			 /* For the sake of coding convenience */
	GETHARDWARETYPE_FRAME_OUT _OUT;
} IOCTL_GETHARDWARETYPE_FRAME, *PIOCTL_GETHARDWARETYPE_FRAME;

typedef struct _IOC_PKG_GETHARDWARETYPE {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_GETHARDWARETYPE_FRAME gethdw;
} IOC_PKG_GETHARDWARETYPE, *PIOC_PKG_GETHARDWARETYPE;


/* Driver handle it */
typedef struct _ADAPTERCOUNT_FRAME_OUT {
	BYTE nCount;
} ADAPTERCOUNT_FRAME_OUT;
typedef union {
	PLACEHOLDER _IN; /* This is just place holder.         */
			 /* For the sake of coding convenience */
	ADAPTERCOUNT_FRAME_OUT _OUT;
} IOCTL_ADAPTERCOUNT_FRAME, *PIOCTL_ADAPTERCOUNT_FRAME;

typedef struct _IOC_PKG_ADAPTERCOUNT {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ADAPTERCOUNT_FRAME adpcnt;
} IOC_PKG_ADAPTERCOUNT, *PIOC_PKG_ADAPTERCOUNT;


/* Driver handle it */
typedef struct _ADAPTER_FRAME_IN {
	ULONG Index;
} ADAPTER_FRAME_IN;
typedef struct _ADAPTER_FRAME_OUT {
	CONTROLLERINFO controller;			
} ADAPTER_FRAME_OUT;
typedef union {
	ADAPTER_FRAME_IN _IN;
	ADAPTER_FRAME_OUT _OUT;
} IOCTL_ADAPTER_FRAME, *PIOCTL_ADAPTER_FRAME;

typedef struct _IOC_PKG_ADAPTER {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ADAPTER_FRAME adp;
} IOC_PKG_ADAPTER, *PIOC_PKG_ADAPTER;


/*
 * Driver handle this IOCTL to provide one ATAPI device information on
 * given Channel ID.
 */
typedef struct _ATAPIDEVICEINFO_FRAME_IN {
	BYTE bChannelID;
	BYTE bID;      /* Master or slave (0 or 1) */
} ATAPIDEVICEINFO_FRAME_IN;
typedef struct _ATAPIDEVICEINFO_FRAME_OUT {
	ULONG ulDevType;     /* defined as ATA spec. ? */
	BYTE bModuleNumber[MAX_MODULE_LENGTH];
	BYTE bSerialNumber[MAX_SERIAL_LENGTH];
	ULONG ulTransferMode;  /* defined at top of this file (convert to enumTransferMode). */
} ATAPIDEVICEINFO_FRAME_OUT;
typedef union {
	ATAPIDEVICEINFO_FRAME_IN _IN ;
	ATAPIDEVICEINFO_FRAME_OUT _OUT;
} IOCTL_ATAPIDEVICEINFO_FRAME, * PIOCTL_ATAPIDEVICEINFO_FRAME;

typedef struct _IOC_PKG_ATAPIDEVICEINFO {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ATAPIDEVICEINFO_FRAME atapi;
} IOC_PKG_ATAPIDEVICEINFO, *PIOC_PKG_ATAPIDEVICEINFO;


/******************EVENT Operation****BEGIN************************/

/*
 * LOGIC:
 * API uses these 2 IOCTLs to get events. First call IOCTL_GETLOGCOUNT_FRAME
 * to get the approximate number of events, then call IOCTL_GETLOGEVENT_FRAME 
 * to get the events. The actually fetched events number is filled in field 
 * IOCTL_GETLOGEVENT_FRAME._IN.pulEvtCount.
 * 
 * Driver moves the pointer of ENGINE EVENTs BUFFER QUEUE, FIFO, firstly rised event firstly fetched.
 *
 *     **********The flow of EVENT GETTING***************       
 *     
 *     #define MAX_EVENT_ONETIME        20
 *     loop:
 *
 *          set loop_flag to FALSE
 *          call IOCTL_GETLOGCOUNT_FRAME          
 *
 *          if IOCTL_GETLOGCOUNT_FRAME._OUT.ulEvtCount>MAX_EVENT_ONETIME then
 *             set loop_flag to TRUE
 *          end if
 *
 *          call IOCTL_GETLOGCOUNT_FRAME
 *          (*IOCTL_GETLOGEVENT_FRAME._IN.pulEvtCount) is really fetched events count
 *
 *          if  loop_flag then
 *              goto loop
 */      

typedef struct _GETLOGCOUNT_FRAME_IN{
		ULONG ulAdapterID;
} GETLOGCOUNT_FRAME_IN;
typedef struct _GETLOGCOUNT_FRAME_OUT{
		ULONG ulEvtCount;
} GETLOGCOUNT_FRAME_OUT;
typedef union {
	GETLOGCOUNT_FRAME_IN _IN;
	GETLOGCOUNT_FRAME_OUT _OUT;
} IOCTL_GETLOGCOUNT_FRAME, *PIOCTL_GETLOGCOUNT_FRAME;

typedef struct _IOC_PKG_GETLOGCOUNT {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_GETLOGCOUNT_FRAME logcnt;
} IOC_PKG_GETLOGCOUNT, *PIOC_PKG_GETLOGCOUNT;



typedef struct _GETLOGEVENT_FRAME_IN {
	ULONG   ulAdapterID;
	ULONG   ulEvtCount;   /* This parameter is IN/OUT, first driver read it to determine */
	                      /* how many events to be fetched, and when event copying finished, */
	                      /* Driver sets this field to actually copied events count. */
	/*void * pBuf; */
} GETLOGEVENT_FRAME_IN;
typedef struct _GETLOGEVENT_FRAME_OUT {
	ULONG   ulEvtCount;   /* Real copied events number */
	BYTE    Buf[MAX_EVENTS_BUF]; 
} GETLOGEVENT_FRAME_OUT;
typedef union {
	GETLOGEVENT_FRAME_IN  _IN;
	GETLOGEVENT_FRAME_OUT _OUT;
} IOCTL_GETLOGEVENT_FRAME, *PIOCTL_GETLOGEVENT_FRAME;

typedef struct _IOC_PKG_GETLOGEVENT {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_GETLOGEVENT_FRAME logevt;
} IOC_PKG_GETLOGEVENT, *PIOC_PKG_GETLOGEVENT;

/*******************EVENT Operation****END**************************/


/********************ROM Operation******BEGIN***********************/

/*#define IOCTL_READ_ROM_INFO  */
/*#define IOCTL_READ_ROM_DATA  */      
/*#define IOCTL_WRITE_ROM_DATA */


/*
 * The LOGIC of ROM operation (updata ROM or read and save ROM to .BIN file):
 *
 * Read process flow of RAIDAPI side:
 *   Read ROM information(header) from Driver (IOCTL_READ_ROM_INFO & IOCTL_ROM_INFO_FRAME)
 *   Read ROM image from Driver (IOCTL_READ_ROM_DATA & IOCTL_ROM_DATA_FRAME)
 *   Verify the check sum of ROM image 
 *   Save it as one file (.BIN file) if verify successful
 *
 * Compare and check ROM header information before RAIDAPI write ROM data to Driver:
 *   Read ROM information(header) from .BIN file 
 *   Read ROM information(header) from Driver (IOCTL_READ_ROM_INFO & IOCTL_ROM_INFO_FRAME)
 *   Get data from above process,Compare version and check VenderID and DeviceID (check valid adapter type)
 *   Verify the check sum of ROM image file (.BIN file)
 *
 * Write process flow of RAIDAPI side if verify successful(above process): 
 *   Write ROM image data (IOCTL_WRITE_ROM_DATA & IOCTL_ROM_DATA_FRAME)
 *
 *
 * Driver side:   
 *   Read ROM header information and give it to RAIDAPI if RAIDAPI demand to read ROM header information(IOCTL_READ_ROM_INFO & IOCTL_ROM_INFO_FRAME)
 *   Read ROM data and give it to RAIDAPI if RAIDAPI demand to read ROM data(IOCTL_READ_ROM_DATA & IOCTL_ROM_DATA_FRAME)
 *   Write ROM image data if RAIDAPI demand to write ROM image data(IOCTL_WRITE_ROM_DATA & IOCTL_ROM_DATA_FRAME)
 */


/*
 * According to Carmel BIOS header information:
 * Read 2 bytes content from ((*(BAR#3+BIOS_INFO_ADDR_POINTER))+BIOS_INFO_OFFSET) will get venderID, then 2 bytes are deviceID.
 * Read 4 bytes from (*(BAR#3+BIOS_VER_ADDROFFSET)) will be version, for example, 1.0.0.7
 * NOTE: If read from binary file, the base address of BIOS code is 0.
 */
#define BIOS_INFO_ADDR_POINTER   0x18
#define BIOS_INFO_ADDR_LEN       2   /* 2 bytes of address. */
#define BIOS_INFO_OFFSET         4
#define BIOS_INFO_ID_LEN         2
#define BIOS_VER_ADDROFFSET      0x2A
#define BIOS_VER_LEN             4
#define BIOS_LEN_OFFSET          2
#define BIOS_LEN_LEN             1   /* BIOS length takes up 1 byte, unit is sector. */

#define ROMTYPE_BIOS		1
#define ROMTYPE_FW		2
#define ROMTYPE_PATCHCODE	3
#define ROMTYPE_HOSTCODE        4


#define BIOS_CHECKSUM            0
#define BIOS_MAX_SIZE            (64*1024) // 64KBytes

#define FWVECTOR_OFFSET          0x0003FF80
#define FWVECTOR_SIZE            128  // Bytes

/*
 * BIOS (or FirmWare) header information
 */
typedef struct _BIOS_INFO
{
	ULONG AdapterID;    /* Support multiple adapters  */
	BYTE  VenderID[BIOS_INFO_ID_LEN];
	BYTE  DeviceID[BIOS_INFO_ID_LEN];
	BYTE  Version[BIOS_VER_LEN];
	ULONG Length;       /* Length of the BIOS binary code, unit is sector.*/
} BIOS_INFO;
typedef union _ROM_INFO {
	BIOS_INFO bios;	
	/* FirmWare can be placed here	*/
} ROM_INFO;
typedef struct _IN_PARA {
	BYTE   AdapterID;
	BYTE   Type;  /* BIOS or Firmware, etc. */
} IN_PARA;
typedef union {
	IN_PARA _IN;   
	ROM_INFO _OUT;
} IOCTL_ROM_INFO_FRAME, * PIOCTL_ROM_INFO_FRAME;

typedef struct _IOC_PKG_ROM_INFO {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ROM_INFO_FRAME rominfo;
} IOC_PKG_ROM_INFO, *PIOC_PKG_ROM_INFO;

/*
 * The ROM image data information
 */

typedef struct _ROM_DATA
{
	USHORT AdapterID; /* Support multiple adapters */
	USHORT ROMType;   /* BIOS or Firmware, etc.  */
	ULONG  TotalSize; /* Total size of ROM image data. Unit is byte. */
	ULONG  StartLoc;  /* Start location.The address of this transfer in whole ROM. */
	                  /* When RAIDAPI read data from Driver, if the StartLoc is 0, then this is first read process. */
	ULONG  Size;      /* The size of valid ROM data in this transfer, Unit is byte. */
	BYTE   buf[16*1024];
	/*BYTE   buf[4*1024-sizeof(SRB_IO_CONTROL)-sizeof(IODATA_HEADER)-2*sizeof(USHORT)-2*sizeof(ULONG)];*/
	                  /* Ensure that this entire structure is in size of 4K Bytes.  */
} ROM_DATA;

/* Both read and write ROM data use this one FRAME structure. */
typedef union {
	ROM_DATA _IN;
	ROM_DATA _OUT;
} IOCTL_ROM_DATA_FRAME, *PIOCTL_ROM_DATA_FRAME;

typedef struct _IOC_ROM_DATA {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ROM_DATA_FRAME romdata;
} IOC_PKG_ROM_DATA, *PIOC_PKG_ROM_DATA;


// Read/Write FW Vector IOCTL frame
typedef struct _FWVECTOR 
{
	BYTE   AdapterID;   // Support multiple adapters
	BYTE   ReadFlag;    // If it is 1, then read FW vector, otherwise 0, write FW vector.
	USHORT Size;        // The size of valid ROM data in this transfer, Unit is byte.
	ULONG  StartLoc;    // Start location.The address of this transfer in whole ROM(V850) space.
	BYTE   buf[FWVECTOR_SIZE];
} FWVECTOR;
// Both read and write FW vector use this one FRAME structure.
typedef union {
	FWVECTOR _IN;
	FWVECTOR _OUT;
} IOCTL_FWVECTOR_FRAME, *PIOCTL_FWVECTOR_FRAME;

typedef struct _IOC_PKG_FWVECTOR {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_FWVECTOR_FRAME romerase;
} IOC_PKG_FWVECTOR, *PIOC_PKG_FWVECTOR;


typedef struct _ROM_ERASE_IN
{
 USHORT AdapterID;
 USHORT ROMType;
} ROM_ERASE_IN;

typedef struct _ROM_ERASE_OUT
{
 ULONG EraseRtnStatus;   /* At present, this value not used, and driver not fill it. */
                         /* IOCTL calling successful already indicates ROM is Erased. */
} ROM_ERASE_OUT;

typedef union {
 ROM_ERASE_IN _IN;
 ROM_ERASE_OUT _OUT;
} IOCTL_ROM_ERASE_FRAME, *PIOCTL_ROM_ERASE_FRAME;


typedef struct _IOC_PKG_ROM_ERASE {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	IOCTL_ROM_ERASE_FRAME romerase;
} IOC_PKG_ROM_ERASE, *PIOC_PKG_ROM_ERASE;

/*******************ROM Operation******END**************************/



/******************SMART Operation*****BEGIN************************/

/* SMART flag definition */
#define SMART_FLAG_UNKNOWN		0x00
#define SMART_FLAG_NOT_SUPPORT		0x01
#define SMART_FLAG_DISABLE		0x02
#define SMART_FLAG_ENABLE		0x03

#define SMART_FLAG_THESHOLD_EXCEEDED	0x04
#define SMART_FLAG_ATTR_AUTOSAVE_ENABLE	0x08


/* WIN_SMART sub-commands */
#define SMART_READ_VALUES 0xD0 /*Has SMART OUT DATA buffer. PIO-IN*/
                               /*Read SMART data structure(one sector) to host*/
#define SMART_READ_THRESHOLDS 0xD1  /* Obsolete */
#define SMART_ATTR_AUTOSAVE   0xD2  /* Enable: 0xF1 Disable: 0x00 
                                     * Enable or disable the SMART 
				     * attributes auto-save */
#define SMART_ATTR_SAVE       0xD3  /* Immediately save any updated 
                                     * SMART attribute values. */
#define SMART_IMMEDIATE_OFFLINE	0xD4
#define SMART_READ_LOG_SECTOR	0xD5  /* Has SMART OUT DATA buffer. PIO-IN 
                                       * Read SMART error log(multi-sectors
                                       * or not) to host.                  
                                       * Data structure: error log sector  
                                       * and SMART Log Directory, and so on.
                                       * See Spec.                          */
#define SMART_WRITE_LOG_SECTOR	0xD6  /* Has SMART IN DATA buffer. PIO_OUT  */
                                      /* Write indicated number of 512bytes */
                                      /* to indicated log.  */
#define SMART_WRITE_THRESHOLDS	0xD7  /* Obsolete */
#define SMART_ENABLE            0xD8  /* Enable SMART */
#define SMART_DISABLE           0xD9  /* Disable SMART */
#define SMART_STATUS            0xDA  /* Return status: Threshold exceed 
                                       * condition FALSE
                                       * Threshold exceed condition TRUE    */
#define SMART_AUTO_OFFLINE      0xDB  /* Obsolete */

/* Parameter for SMART_ATTR_AUTOSAVE */
/* Convert to one byte length */
typedef enum _PARAM_ATTR_AUTOSAVE {
	AttrAutoEnable     =  0xF1, /* Enable the SMART attributes auto-save */
	AttrAutoDisable    =  0x00  /* Disable */
} PARAM_ATTR_AUTOSAVE;

/* Parameter for SMART_IMMEDIATE_OFFLINE */
/* Covert to one byte length */
typedef enum _PARAM_OFFLINE {
	OffModeOfflineRut       =  0,  /* Execute off-line routine immediately in off-line mode */
	OffModeShortSelfTstRut  =  1,  /* Execute short self-test routine immediately in off-line mode */
	OffModeExtSelfTstRut    =  2,  /* Execute extended self-test routine immediately in off-line mode */
	AbortOffModeSelfTstRut  = 127, /* Abort offline mode selftest routine.*/
	CapModeShortSelfTstRut  = 129, /* Execute short self-test routine immediately in captive mode */
	CapModeExtSelfTstRut    = 130  /* Execute extended self-test routine immediately in captive mode */
} PARAM_OFFLINE;

/* Parameter for SMART_READ_VALUES */
#define LOGADDR_Dir         0x00 /* Log directory */
#define LOGADDR_SumErrLog   0x01 /* Summary SMART error log */
#define LOGADDR_CompErrLog  0x02 /* Comprehensive SMART error log */
#define LOGADDR_SelfTstLog  0x06 /* SMART self-test error log */

typedef struct _PARAM_RAEADVALUS {
	BYTE    SectNum; /* Number of sectors to be read.(Index or Count???) */
	BYTE    LogAddr; /* Value defined as above */
} PARAM_READLOG;

/* SMART_READ_VALUES doesn't need additional parameter. */

typedef union _SMARTCMD_PARAM
{
	PARAM_ATTR_AUTOSAVE auto_save;
	PARAM_OFFLINE       offline;
	PARAM_READLOG       read_log;
	ULONG               placeholder; /* Never use it, */
	                                 /* Keep it as placeholder to retain 4bytes alignment. */
	                                 /* Should alway keep its size as multiple of 4. */
} SMARTCMD_PARAM;

typedef struct _SMARTDATA_OUT_PARA {
 ULONG  TotalSize;  /* Total size of data. Unit is byte. */
 ULONG  StartLoc;   /* Start location.The address of this transfer in whole process */
 ULONG  Size;       /* The size of valid data in this transfer, Unit is byte. */
 BYTE   buf[512];
} SMARTDATA_OUT_PARA; 

typedef struct _SMART_OUT_PARA {
 USHORT StatusFlag; /* All SMART Status and Flag indicator. 
                     * See SMART flag definition above. 
                     * bit0 and bit1: (SMART_FLAG_ENABLE is prerequisite for
                     *                 other bits' validity.)
                     *     0: SMART_FLAG_UNKNOWN, 1: SMART_FLAG_NOT_SUPPORT
                     *     2: SMART_FLAG_DISABLE, 3: SMART_FLAG_ENABLE 
                     * bit2: Threshold status.
                     *     0: Threshold not exceed, 1: Threshold exceed.
                     * bit3: Attribute auto-save.
                     *     0: attribute auto-save disabled.
                     *     1: attribute auto-save enabled.  */
} SMART_OUT_PARA; 

typedef struct _SMARTDATA_IN_PARA  {
 BYTE            AdapterID;
 BYTE            Channel;
 BYTE            IDESubCmd;
 BYTE            Reserved;   /* Reserved for byte alignment */
 SMARTCMD_PARAM  Param;      /* Parameter for given SMART sub-command. If not need parameter, skip it. */
 ULONG           TotalSize;  /* Total size of data. Unit is byte. */
 ULONG           StartLoc;   /* Start location.The address of this transfer in whole process */
 ULONG           Size;       /* The size of valid data in this transfer, Unit is byte. */
 BYTE            buf[512];
} SMARTDATA_IN_PARA;

typedef struct _SMART_IN_PARA {
 BYTE            AdapterID;
 BYTE            Channel;
 BYTE            IDESubCmd;
 BYTE            Reserved;   /* Reserved for byte alignment. 
                              * Driver should not use this byte as addtional
                              * parameter for SMART_ATTR_AUTOSAVE shown as 
                              * following: */
 SMARTCMD_PARAM  Param;      /* Parameter for given SMART sub-command. */
                             /* If not need parameter, skip it. */
} SMART_IN_PARA;

/* See IODATA_HEADER.SMARTDataBufFlag to check wether the Frame has IN or OUT DATA buffer. */

/* Has no in data buffer and has no out data buffer */
typedef union {
 SMART_IN_PARA    _IN;   
 SMART_OUT_PARA   _OUT;
} IOCTL_SMART00_FRAME, * PIOCTL_SMART00_FRAME;

/* Has in data buffer but has no out data buffer */
typedef union {
 SMARTDATA_IN_PARA  _IN;   
 SMART_OUT_PARA     _OUT;
} IOCTL_SMART01_FRAME, * PIOCTL_SMART01_FRAME;

/* Has no in data buffer but has out data buffer */
typedef union {
 SMART_IN_PARA       _IN;   
 SMARTDATA_OUT_PARA  _OUT;
} IOCTL_SMART10_FRAME, * PIOCTL_SMART10_FRAME;

/* Both Has in data buffer and out data buffer */
typedef union {
 SMARTDATA_IN_PARA   _IN;   
 SMARTDATA_OUT_PARA  _OUT;
} IOCTL_SMART11_FRAME, * PIOCTL_SMART11_FRAME;

typedef struct _IOC_PKG_SMART {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	union {
		IOCTL_SMART00_FRAME smart00;
		IOCTL_SMART01_FRAME smart01;
		IOCTL_SMART10_FRAME smart10;
		IOCTL_SMART11_FRAME smart11;
	} smart;
} IOC_PKG_SMART, *PIOC_PKG_SMART;

typedef struct _PASSTHRU_REG_IN {
	U8 bLBAWidth;
	U8 bCmdProtocol;
	U8 bReserved1;
	U8 bReserbed2;
	struct {
		U8 bFeature;
		U8 bSectorCounts;
		U8 bLBAHigh;
		U8 bLBAMid;
		U8 bLBALow;
		U8 bDevice;
		U8 bCommand;
		U8 bControl;
	} Current;
	struct {
		U8 bFeature;
		U8 bSectorCounts;
		U8 bLBAHigh;
		U8 bLBAMid;
		U8 bLBALow;
		U8 bReserved3;
		U8 bReserved4;
		U8 bControl; 
	} Previous;
} PASSTHRU_REG_IN, *PPASSTHRU_REG_IN;

typedef struct _PASSTHRU_REG_OUT {
	U8 bReserved1;
	U8 bReserved2;
	U8 bReserved3;
	U8 bReserbed4;
	struct {
		U8 bError;
		U8 bSectorCounts;
		U8 bLBAHigh;
		U8 bLBAMid;
		U8 bLBALow;
		U8 bDevice;
		U8 bStatus;
		U8 bAlternateStatus;
	} Current;
	struct {
		U8 bError;
		U8 bSectorCounts;
		U8 bLBAHigh;
		U8 bLBAMid;
		U8 bLBALow;
		U8 bDevice;
		U8 bStatus;
		U8 bAlternateStatus; 
	} Previous;
} PASSTHRU_REG_OUT, *PPASSTHRU_REG_OUT;

typedef union _PASSTHRU_CMD_DATA {
	PASSTHRU_REG_IN _IN;
	PASSTHRU_REG_OUT _OUT;
} PASSTHRU_CMD_DATA, *PPASSTHRU_CMD_DATA;


/******************SMART Operation****END**************************/


/******************Enclosure Operations**BEGIN*********************/

/* 
 * Device ID definition
 *  (Refer to: PSS10398A Enclosure chip For SATA Controller Development Guid.)
 */
#define SS_DEVID_MASK               0x1F
#define SS_DEVID_1100_1039A_1F1S    0x00  /* 000 00 */
#define SS_DEVID_4100_1039A_2F2S    0x09  /* 010 01 */
#define SS_DEVID_OEM1CH_NOPWM_1F1S  0x02  /* 000 10 */
#define SS_DEVID_NECSATA_BP         0x07  /* 001 11 */
#define SS_DEVID_OEM4CH_NOPWM_2F2S  0x0B  /* 010 11 */
#define SS_DEVID_OEM4CH_NOPWM_3F3S  0x1F  /* 111 11 */
#define SS_DEVID_INVALID            0xFF  /* (-1) */

#define MAX_SS_NUMBER              MAX_CHANNEL_PER_ADAPTER /* For 8 channels product, there can be 8 SS1100 */
#define MAX_SS_ENCBOX_NUMBER       4  /* Max drive carrier number on one super swap ss4100 */
#define MAX_SS_FAN_NUMBER          3  /* OEM 4channels super swap */
#define MAX_SS_TEMPSENSOR_NUMBER   3  /* OEM 4channels super swap */

#define ENC_CMD_ALLCONFIG          0x00
#define ENC_CMD_FAN                0x01
#define ENC_CMD_SENSOR             0x02

typedef struct _ENC_IN_PARA {
	BYTE    AdapterID; /* Support multiple cards. */
	BYTE    Cmd;       /* ENC_CMD_FAN/ENC_CMD_SENSOR, Read Fan or Read Sensor. */
	USHORT  Index;     /* When Cmd not equal to ENC_CMD_ALLCONFIG, then this field is valid. */
	                   /* 0:Sensor#1 or Fan#1, 1: Sensor#2 or Fan#2, 2: Sensor#3 or Fan#3 */
} ENC_IN_PARA;
typedef struct _ENC_OUT {
	BYTE    EncPwrStat; /* Bit0 represents whether SMBusID=0 has disk inserted. etc. */
	BYTE    EncType[MAX_SS_NUMBER]; /* Every EncBox type. */
	BYTE    DiskID[MAX_SS_NUMBER];  /* Map between SMBusID and diskID. If there is no disk on one SMBusID, then DiskID = -1; */

	ULONG   F1[MAX_SS_NUMBER];
	ULONG   S1[MAX_SS_NUMBER];
	ULONG   V33[MAX_SS_NUMBER];
	ULONG   V5[MAX_SS_NUMBER];
	ULONG   V12[MAX_SS_NUMBER];
} ENC_OUT;
typedef struct _ENC_OUT2 {
	ULONG   Val[MAX_SS_NUMBER]; /* Read Fan#1,Fan#2, Fan#3, Sensor#1,Sensor#2,Sensor#3 */
} ENC_OUT2;
typedef union {
	ENC_IN_PARA _IN;
	ENC_OUT     _OUT;
} IOCTL_ENC_FRAME, *PIOCTL_ENC_FRAME;
typedef union {
	ENC_IN_PARA _IN;
	ENC_OUT2    _OUT;
} IOCTL_ENC_FRAME2, *PIOCTL_ENC_FRAME2;

typedef struct _IOC_PKG_ENC {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	union {
		IOCTL_ENC_FRAME   enc;
		IOCTL_ENC_FRAME2  enc2;
	} enclosure;
} IOC_PKG_ENC, *PIOC_PKG_ENC;

/******************Enclosure Operations**END***********************/



/*******************Payload Style IOCTLs****BEGIN******************/



/* According to MessagePayload Get Array Description */
typedef struct _ARRAY_FRAME_IN {
		USHORT usOPCode;          /* Message unit OP code: 0x0008 */
		BYTE  bAryID; 
		BYTE  Reserved;           /* Message unit place holder */
		ULONG ulReturnHandle;     /* Message unit place holder */
		ULONG OutputPointer;      /* Message unit place holder */
		ULONG ulReserved;         /* Message unit place holder */
} ARRAY_FRAME_IN;
typedef struct _ARRAY_FRAME_OUT {
		ULONG  ulArySize;
		USHORT usArySizeHigh;
		USHORT usStripeSize;
		ULONG  ulRAIDMode;
		ULONG  ulStripeBlockSize;
		USHORT usCyls;
		USHORT usHeads;
		USHORT usSectors;
		USHORT usReserved;
		BYTE   bName[MAX_ARY_NAME_LENGTH];
		ULONG  ulAryStatus;
		BYTE   nDisks;  /* number of disks */
		BYTE   DiskID[MAX_DISK_PER_ARRAY];  /* bit7..4:	logic stream number from 0 to 0x7 */
	                                        /* bit3..0:	device number from 0 to 0xF (2*channelID) */
} ARRAY_FRAME_OUT;
#define MSG_OPCODE_ARRAY          0x0008
typedef union {
	 ARRAY_FRAME_IN _IN;
	 ARRAY_FRAME_OUT _OUT;	
} IOCTL_ARRAY_FRAME, *PIOCTL_ARRAY_FRAME;


/* According to MessagePayload Get Disk Description */
typedef struct _DISK_FRAME_IN {
		USHORT  usOPCode;               /* Message OP code: 0x0007 */
		BYTE    bDevID;                 /* ChannelID*2+ID  */
		BYTE    fIdentifyData;          /* Flag: 1= Transfer Identift data, 0 not */
		                                /* Here should be 0 */
		ULONG   ulReturnHandle;         /* Message unit place holder */
		ULONG   ulOutputPointer;        /* Message unit place holder */
		ULONG   ulReserved;             /* Message unit place holder */
} DISK_FRAME_IN;
typedef struct _DISK_FRAME_OUT {
		ULONG   ulDiskStatus;
		BYTE    bAryID;
		BYTE    Reserved[3];
		ULONG   ulDiskSize;
		USHORT  usDiskSizeHigh;
		USHORT  usDiskBlockSize;
} DISK_FRAME_OUT;
typedef struct {
		ULONG   ulDiskStatus;
		BYTE    bAryID;
		BYTE    Reserved[3];
		ULONG   ulDiskSize;
		USHORT  usDiskSizeHigh;
		USHORT  usDiskBlockSize;
		BYTE    IdentifyData[IDENTIFY_DATA_LENGTH];
} DISK2_FRAME_OUT;
#define MSG_OPCODE_DISK                 0x0007
typedef union 
{
	 DISK_FRAME_IN _IN ;
	 DISK_FRAME_OUT _OUT;
} IOCTL_DISK_FRAME, *PIOCTL_DISK_FRAME;

/* Message same to IOCTL_DISK_FRAME but contain Identify data */
typedef union 
{
	 DISK_FRAME_IN _IN ;
	 DISK2_FRAME_OUT _OUT;
} IOCTL_DISK2_FRAME, *PIOCTL_DISK2_FRAME;




/*
 * Use following IOCTLs to get channel info.
 */
typedef struct _SCANCHANNEL_FRAME_IN {
		USHORT  usOPCode;               /* Message OP code: 0x0506 */
		BYTE    bAdapterID;             
		BYTE    Reserved;               /* Message unit place holder */
		ULONG   ulReturnHandle;         /* Message unit place holder */
		ULONG   ulOutputPointer;        /* Message unit place holder */
		ULONG   ulReserved;             /* Message unit place holder */
} SCANCHANNEL_FRAME_IN;
typedef struct _SCANCHANNEL_FRAME_OUT {
		/* If the device type is not 0x00 and 0x1F then ATAPI device. */
		BYTE bDevType[MAX_CHANNEL_PER_ADAPTER]; /* Here Channel type same to device type */
} SCANCHANNEL_FRAME_OUT;
#define MSG_OPCODE_SCANNCHANNEL         0x0506
typedef union {
	 SCANCHANNEL_FRAME_IN _IN ;
	 SCANCHANNEL_FRAME_OUT _OUT;
} IOCTL_SCANCHANNEL_FRAME, * PIOCTL_SCANCHANNEL_FRAME;



typedef struct _CREATEARRAY_FRAME_IN {
		USHORT usOPCode; /* Message OP Code: 0x0108 */
	
		BYTE   Reserved[2];       /* Message Unit place holder */
		ULONG  ulReturnHandle;    /* Message Unit place holder */
		ULONG  ulOutputPointer;   /* Message Unit place holder */
			
		ULONG  ulRAID;            /* bit flag of RAID type */
		                          /* BIT31: 1=do initialization; 0=not do initializing */
		                          /* Bit30:	1=Gigabyte boundary; 0=not */
		                          /* Bit29:	If (bit31==1) 1=background initializing; 0=foreground initializing. */
		                          /* Bit28:	If (bit31==1) 1=full initializing; 0=quick initializing */
                                  /* Bit27:	1=do duplicating (rebuild) creating array; 0=create only (not do duplicating) */
                                  /* Bit26..8:	reserved */
                                  /* Bit7..0:	0x0	= RAID0 */
	                              /*            0x1	= RAID1 */
	                              /*            0x2	= JBOD */
	                              /*            0x4	= RAID5 */
		USHORT StripeBlockSize;   /* StripeBlockSize is the number of sectors. */
		/* The source disks' channel ID, maximum 4 disks to be duplicated. */
		USHORT SourceChannel;     /* if (bit27==1) indicates source disk which contain user data for duplicating when creating.  */
		                          /* Each 4 bits for one channelID, bit3..0 for channelID[0], bit7..4 for channelID[1]... */
		char   AryName[MAX_ARY_NAME_LENGTH];
		ULONG  ulReserved;
		BYTE   nDisk;
		BYTE   DiskID[MAX_DISK_PER_ADAPTER];
} CREATEARRAY_FRAME_IN;
typedef struct _CREATEARRAY_FRAME_OUT {
		ULONG  ArySize;
		USHORT ArySizeHigh;
		USHORT StripeSize;
		ULONG  RAIDMode;
		USHORT StripeBlockSize;
		USHORT Reserved1;
		USHORT Cyls;
		USHORT Headers;
		USHORT Sectors;
		BYTE   AryID;
		BYTE   Reserved2;
		char   AryName[MAX_ARY_NAME_LENGTH];
		ULONG  AryStatus;
		BYTE   nDisk;
		BYTE   DiskID[MAX_DISK_PER_ARRAY];  /* bit7..4:	logic stream number from 0 to 0x7 */
	                                        /* bit3..0:	device number from 0 to 0xF (2*channelID) */
} CREATEARRAY_FRAME_OUT;
#define MSG_OPCODE_CREATEARRAY      0x0108
typedef union {
	 CREATEARRAY_FRAME_IN _IN;
	 CREATEARRAY_FRAME_OUT _OUT;
} IOCTL_CREATEARRAY_FRAME, *PIOCTL_CREATEARRAY_FRAME;





typedef struct _DELETEARRAY_FRAME_IN {
		USHORT usOPCode; /* Message OP Code: 0x0208 */
		BYTE  Index;
		BYTE  Reserved;
		ULONG ulReturnHandle;  /* Message Unit place holder */
		ULONG ulReserved[2];   /* Message Unit place holder */
} DELETEARRAY_FRAME_IN;
#define MSG_OPCODE_DELETEARRAY       0x0208
typedef union {
	 DELETEARRAY_FRAME_IN _IN;
	 PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_DELETEARRAY_FRAME, *PIOCTL_DELETEARRAY_FRAME;




typedef struct _RENAMEARRAY_FRAME_IN {
		USHORT usOPCode; /* Message OP Code: 0x0908 */
		BYTE   Index;
		BYTE   Reserved;
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		ULONG  ulReserved;      /* Message Unit place holder */
		char   AryName[MAX_ARY_NAME_LENGTH];
} RENAMEARRAY_FRAME_IN;
#define MSG_OPCODE_RENAMEARRAY          0x0908
typedef union {
	RENAMEARRAY_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_RENAMEARRAY_FRAME, *PIOCTL_RENAMEARRAY_FRAME;




/* About fixed stream:
It is a mapping to indicate which disk (source or mirror) will be fixed by another (mirror or source). 
In an array (only for RAID1 or RAID0+1), only the last significant N (N is the number of disks in the array) 
bits are used, so N should be an even number. For example, in an array with 6 disks N=6, in an array with 4 disks N=4.
Bit n (0 <= n < N) indicates whether stream n in an array will be fixed from corresponding disk. 
The stream number of a disk in the array can be got from output data in the message "array description".

Stream n and stream ((n + N/2)%N) are a pair disks. If (bit n ==1 ) stream n will be fixed from stream ((n + N/2)%N). 
So only one of bit n and bit ((n + N/2)%N) can be set to one.
For example, in an array (RAID 0 + 1) with 6 disks, N=6, Fixed stream = 0x15 (or 10101 in binary, stream number from 0 to 5):
stream 0 (bit0=1) can be fixed from stream 3 (bit3=0)
stream 4 (bit4=1) can be fixed from stream 1 (bit1=0)
stream 2 (bit2=1) can be fixed from stream 5 (bit5=0)
*/

/*
 * This message is for CheckIntegrity and Synchronize, using BYTE bForceWrite
 * to distinguish. 
 * Synchronize also can be activated when create a new array.
 */
typedef struct _SYNCARRAY_FRAME_IN {
		USHORT usOPCode;        /* Message OP Code: 0x0608 */
		BYTE   Index;
		BYTE   bForceWrite;     /* Flag, if it is 0 then check integrity, */
		                        /* otherwise synchronize.*/
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		/*ULONG  ulReserved[2]; */   /* Message Unit place holder */
		ULONG  ulStartLBA;
		USHORT usStartLBAHigh;
		/*ULONG  ulReserved; */
		BYTE   FixedStream;     /* Only when bForceWrite==-1 (Sync.) then this filed valid */
		                        /* This is only for RAID0+1 RAID mode */
		                        /* And the Stream Mapping is designed by user. */
		                        /* The stream pair according to : stream n and stream (n+N/2)%N are one pair */
		BYTE   Reserved;
} SYNCARRAY_FRAME_IN;

#define MSG_OPCODE_SYNCARRAY    0x0608
typedef union {
	SYNCARRAY_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_SYNCARRAY_FRAME, *PIOCTL_SYNCARRAY_FRAME;

typedef struct _REBUILDARRAY_FRAME_IN {
		USHORT usOPCode; /* Message OP Code: 0x0708 */
		BYTE   Index;
		BYTE   DiskID;   /* Channel*2+Master/Slave */
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		ULONG  ulReserved[2];   /* Message Unit place holder */
} REBUILDARRAY_FRAME_IN;
#define MSG_OPCODE_REBUILDARRAY       0x0708   
typedef union {
	REBUILDARRAY_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_REBUILDARRAY_FRAME, *PIOCTL_REBUILDARRAY_FRAME;



typedef struct _MIGRATEARRAY_FRAME_IN {
		USHORT usOPCode;          /* Message OP Code: 0x0308 */
		BYTE Index;
		BYTE bRAIDMode;
		ULONG ulReturnHandle;     /* Message Unit place holder */
		ULONG ulStripeBlockSize;  /* Its unit is sector */
						
		BYTE  nDisk;              /* Complete disk number and disk ids of new raid mode */
		/* Channel*2+Master/Slave */
		BYTE  DiskID[MAX_DISK_PER_ADAPTER];
} MIGRATEARRAY_FRAME_IN;
#define MSG_OPCODE_MIGRATEARRAY       0x0308
typedef union {
	MIGRATEARRAY_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_MIGRATEARRAY_FRAME, *PIOCTL_MIGRATEARRAY_FRAME;




typedef struct _MAINTENANCE_FRAME {
		USHORT usOPCode;       /* Message OP Code: 0x0408 */
		BYTE  Index;
		BYTE  Maintenance;     /* 0:stop(pause), 1:resume, 2:abort */
		ULONG ulReturnHandle;  /* Message Unit place holder */
		ULONG ulReserved[2];   /* Message Unit place holder */
} MAINTENANCE_FRAME;
#define MSG_OPCODE_ARY_MAINTENANCE     0x0408
typedef union {
	MAINTENANCE_FRAME _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_ARY_MAINTENANCE_FRAME, *PIOCTL_ARY_MAINTENANCE_FRAME;




/*Array Maintenance Lookup */
typedef struct _ARY_MAINTENANCE_LOOKUP_FRAME_IN {
		USHORT usOPCode; /* Message OP Code: 0x0508 */
		BYTE   Index;
		BYTE   Reserved;
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		ULONG  ulOutputPointer; /* Message Unit place holder */
		ULONG  ulReserved;      /* Message Unit place holder */
} ARY_MAINTENANCE_LOOKUP_FRAME_IN;
typedef struct _ARY_MAINTENANCE_LOOKUP_FRAME_OUT {
		BYTE   nPercentage;
		BYTE   Maintenance;
		/*
		Maintenance:
		bit7: Initializing paused
		bit6: rebuilding paused
		bit5: migrating paused
		bit4: checking integrity paused
		bit3: Initializing
		bit2: rebuilding
		bit1: migrating
		bit0: checking integrity
		Percentage: a number from 0 to 100 indicates maintenance job's progress.
		*/
		BYTE   Reserved[2];
} ARY_MAINTENANCE_LOOKUP_FRAME_OUT;
#define MSG_OPCODE_ARY_MAINTENANCE_LOOKUP  0x0508
typedef union {
	ARY_MAINTENANCE_LOOKUP_FRAME_IN _IN;
	ARY_MAINTENANCE_LOOKUP_FRAME_OUT _OUT;
} IOCTL_ARY_MAINTENANCE_LOOKUP_FRAME, *PIOCTL_ARY_MAINTENANCE_LOOKUP_FRAME;


typedef struct _GET_MAINTENANCE_RATE_FRAME_IN {
		USHORT usOPCode;       /* Message OP Code:  0x0B08 */
		BYTE  Index;
		BYTE  Reserved;
		ULONG ulReturnHandle;  /* Message Unit place holder */
		ULONG ulOutputPointer; /* Message Unit place holder */
		ULONG ulReserved;      /* Message Unit place holder */
} GET_MAINTENANCE_RATE_FRAME_IN;
typedef struct _GET_MAINTENANCE_RATE_FRAME_OUT {
		USHORT Rate;           /* Rate from 0 to 4 */
		USHORT AutoRebuild;    /* 1 auto, 0 manual */
} GET_MAINTENANCE_RATE_FRAME_OUT;

#define MSG_OPCODE_GET_MAINTENANCE_RATE     0x0B08
typedef union {
	 GET_MAINTENANCE_RATE_FRAME_IN _IN;
	 GET_MAINTENANCE_RATE_FRAME_OUT _OUT;
} IOCTL_GET_MAINTENANCE_RATE_FRAME, *PIOCTL_GET_MAINTENANCED_RATE_FRAME;




typedef struct _SET_MAINTENANCE_RATE_FRAME_IN {
		USHORT usOPCode;        /* Message OP Code:  0x0A08 */
		BYTE   Index;
		BYTE   Reserved;
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		USHORT Rate;            /* From 0 to 4 */
		USHORT AutoRebuild;     /* 1 Auto, 0 manual */
		ULONG  ulReserved;      /* Message Unit place holder */
} SET_MAINTENANCE_RATE_FRAME_IN;
#define MSG_OPCODE_SET_MAINTENANCE_RATE     0x0A08
typedef union {
	SET_MAINTENANCE_RATE_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_SET_MAINTENANCE_RATE_FRAME, *PIOCTL_SET_MAINTENANCE_RATE_FRAME;


typedef struct _GET_FIRMVERSION_FRAME_IN {
	USHORT usOPCode;       /* Message OP Code:  0x0209 */
	USHORT Reserved;
	ULONG ulReturnHandle;  /* Message Unit place holder */
	ULONG ulOutputPointer; /* Message Unit place holder */
	ULONG ulReserved;      /* Message Unit place holder */
} GET_FIRMVERSION_FRAME_IN;

typedef struct _GET_FIRMVERSION_FRAME_OUT{
	ULONG ulVersion; /*Bit 2:one for 4-port device; zero for 8-port device*/
	                 /*Bit 1:Raid Feature: zero for RAID mode (support 0, 1, 10, JBOD); one for Pass-through */
	                 /*Bit 0:Zero Channel Raid present or not. 0 for not present; 1 for present.*/
	BYTE  ProductType;    
	BYTE  Reserved[3];
} GET_FIRMVERSION_FRAME_OUT;

#define MSG_OPCODE_GET_FIRMVERSION    0x0209
typedef union {
	GET_FIRMVERSION_FRAME_IN _IN;
	GET_FIRMVERSION_FRAME_OUT _OUT;
} IOCTL_GET_FIRMVERSION_FRAME, *PIOCTL_GET_FIRMVERSION_FRAME;


/*
 * Newly added payload IOCTL Frame
 */
typedef struct _DISK_SET_SPARE_FRAME_IN {
	USHORT  usOPCode;  /* MsgOpCode : 0x0207 */
	BYTE    DevID;     /* Disk ID */
	BYTE    SpareFor;  /* For which array the disk spare
	                    * Bit n is one to indicate that the disk is spare for array n (0 <= n <= 7).
	                    * If it is 0x00 then unspare the disk.
	                    * An Array cann't have more than one spare disk */
	ULONG   ulReturnHandle; /* Message place holder */
} DISK_SET_SPARE_FRAME_IN;

typedef struct _DISK_SET_SPARE_FRAME_OUT {
	BYTE PlaceHolder;  
} DISK_SET_SPARE_FRAME_OUT;

#define MSG_OPCODE_DISK_SET_SPARE             0x0207
typedef union {
	DISK_SET_SPARE_FRAME_IN _IN;
	DISK_SET_SPARE_FRAME_OUT _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_DISK_SET_SPARE_FRAME, *PIOCTL_DISK_SET_SPARE_FRAME;




typedef struct _DISK_GET_SPARE_FRAME_IN {
		USHORT  usOpCode;
		USHORT  Reserved;
} DISK_GET_SPARE_FRAME_IN;
typedef struct _DISK_GET_SPARE_FRAME_OUT {
		BYTE    AryMap[MAX_CHANNEL_PER_ADAPTER];
		                                     /* Every byte corresponding to one channel, i.e., one disk setting. */
} DISK_GET_SPARE_FRAME_OUT;
#define MSG_OPCODE_DISK_GET_SPARE             0x0507       
typedef union {
	DISK_GET_SPARE_FRAME_IN _IN;
	DISK_GET_SPARE_FRAME_OUT _OUT;
} IOCTL_DISK_GET_SPARE_FRAME, *PIOCTL_DISK_GET_SPARE_FRAME;



/* Get/Set SATA Transceiver parameter */
typedef struct _GET_SATA_PARAMETER_FRAME_IN {
		USHORT    usOPCode;                  /* MsgCode: 0x1106 */
		BYTE      ChannelID;
		BYTE      Reserved;
		ULONG     ulReturnHandle;            /* Message Unit place holder */
		ULONG     ulOutputPointer;           /* Message Unit place holder  */
} GET_SATA_PARAMETER_FRAME_IN;
typedef struct _GET_SATA_PARAMETER_FRAME_OUT {
		USHORT    Speed;                     /* Speed is an 8 bits mapping,  */
		                                     /* each bit indicates the speed of corresponding channel. */
		                                     /* 0~2 */
		
		USHORT    PreLevel;                  /* prelevel: 2-bits number indicate preemphasis for each channel,  */
		                                     /* bit1..0 for channel 0, bit 3..2 for channel 1 and so on. */
} GET_SATA_PARAMETER_FRAME_OUT;
#define MSG_OPCODE_GET_SATA_PARAMETER         0x1106
typedef union {
	GET_SATA_PARAMETER_FRAME_IN _IN;
	GET_SATA_PARAMETER_FRAME_OUT _OUT;
} IOCTL_GET_SATA_PARAMETER_FRAME,*PIOCTL_DISK_GET_SATA_PARAMETER_FRAME;




typedef struct _SET_SATA_PARAMETER_FRAME_IN {
		USHORT    usOPCode;                  /* MsgCode: 0x1206 */
		BYTE      ChannelID;
		BYTE      Reserved;
		ULONG     ulReturnHandle;            /* Message Unit place holder */
		ULONG     ulOutputPointer;           /* Message Unit place holder */

		USHORT    Speed;                     /* Speed is an 8 bits mapping, */
		                                     /* each bit indicates the speed of corresponding channel.*/
		                                     /* 0~2*/
		USHORT    PreLevel;                  /* prelevel: 2-bits number indicate preemphasis for each channel, */
} SET_SATA_PARAMETER_FRAME_IN;

#define MSG_OPCODE_SET_SATA_PARAMETER    0x1206
typedef union {
	SET_SATA_PARAMETER_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_SET_SATA_PARAMETER_FRAME, *PIOCTL_DISK_SET_SATA_PARAMETER_FRAME;


/*added by isaac for IOCTL test utiliy */
typedef struct _GET_POWER_STATUS_FRAME_IN {
		USHORT usOPCode;
		USHORT Reserved;
} GET_POWER_STATUS_FRAME_IN;
typedef struct _GET_POWER_STATUS_FRAME_OUT {
		ULONG  PowerStatus;  
} GET_POWER_STATUS_FRAME_OUT;
#define MSG_OPCODE_GET_POWER_STATUS     	 0x0F06
typedef union {
	GET_POWER_STATUS_FRAME_IN _IN;
	GET_POWER_STATUS_FRAME_OUT _OUT; 
}IOCTL_GET_POWER_STATUS_FRAME, *PIOCTL_GET_POWER_STATUS_FRAME;


typedef struct _SET_POWER_STATUS_FRAME_IN {
		USHORT usOPCode;
		USHORT Reserved;
		ULONG ulReturnHandle;  /* Message Unit place holder */
		ULONG ulReserved;      /* Message Unit place holder */
		ULONG PowerStatus;
} SET_POWER_STATUS_FRAME_IN;
#define MSG_OPCODE_SET_POWER_STATUS     	 0x0E06
typedef union {
	SET_POWER_STATUS_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
}IOCTL_SET_POWER_STATUS_FRAME, *PIOCTL_SET_POWER_STATUS_FRAME;


typedef struct _DISK_WIPE_SECTOR_FRAME_IN {
		USHORT usOPCode;
		BYTE	DeviceID;
		BYTE    Sector;
} DISK_WIPE_SECTOR_FRAME_IN;

#define MSG_OPCODE_DISK_WIPE_SECTOR                0x0307
typedef union{
	DISK_WIPE_SECTOR_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
}IOCTL_DISK_WIPE_SECTOR_FRAME, *PIOCTL_DISK_WIPE_SECTOR_FRAME;	
/*isaac end */


#define QDEPTH(depth)          (depth>>1)
#define QENABLED(depth)        (depth&1)
#define QCOMB(enable, depth)   ((enable&1)+(depth<<1))
typedef struct _GETQDEPTH_IN {
		USHORT  usOPCode;
		BYTE	CHID;
		BYTE    Reserved;
		ULONG   ulReturnHandle;  /* Message Unit place holder */
		ULONG   ulOutputPointer; /* Message Unit place holder */
		ULONG   ulReserved;      /* Message Unit place holder */
} GETQDEPTH_IN;
typedef struct _GETQDEPTH_OUT {
	    ULONG   Depth; /* Bit0: 1 for enabled TCQ/NCQ, 0 for disable */
		               /* Bit1~Bit31: Depth value */
} GETQDEPTH_OUT;
#define MSG_OPCODE_GETQDEPTH               0x1306
typedef union {
	GETQDEPTH_IN  _IN;
	GETQDEPTH_OUT _OUT;
} IOCTL_GETQDEPTH_FRAME, *PIOCTL_GETQDEPTH_FRAME;

typedef struct _SETQDEPTH_IN {
		USHORT  usOPCode;
		BYTE	CHID;
		BYTE    Reserved;
		ULONG   ulReturnHandle;  /* Message Unit place holder */
		ULONG   ulReserved;      /* Message Unit place holder */
	    ULONG   Depth;           /* Bit0: 1 for enabled TCQ/NCQ, 0 for disable */
		                         /* Bit1~Bit31: Depth value */
} SETQDEPTH_IN;
#define MSG_OPCODE_SETQDEPTH               0x1406
typedef union {
	SETQDEPTH_IN  _IN;
	PLACEHOLDER   _OUT;
} IOCTL_SETQDEPTH_FRAME, *PIOCTL_SETQDEPTH_FRAME;


typedef struct _IOC_PKG_ARRAY {
	SRB_IO_CONTROL	ioc;
	IODATA_HEADER	head;
	union {
		IOCTL_ARRAY_FRAME		array;
		IOCTL_DISK_FRAME		disk;
		IOCTL_DISK2_FRAME		disk2;
		IOCTL_SCANCHANNEL_FRAME		scanchan;
		IOCTL_CREATEARRAY_FRAME		creatarr;
		IOCTL_DELETEARRAY_FRAME		delarr;
		IOCTL_RENAMEARRAY_FRAME		renamarr;
		IOCTL_SYNCARRAY_FRAME		syncarr;
		IOCTL_REBUILDARRAY_FRAME	rebltarr;
		IOCTL_MIGRATEARRAY_FRAME	migrarr;
		IOCTL_ARY_MAINTENANCE_FRAME	maint;
		IOCTL_ARY_MAINTENANCE_LOOKUP_FRAME	mainlookup;
		IOCTL_GET_MAINTENANCE_RATE_FRAME	getmaintrat;
		IOCTL_SET_MAINTENANCE_RATE_FRAME	setmaintrat;
		IOCTL_GET_FIRMVERSION_FRAME	firmver;
		IOCTL_DISK_SET_SPARE_FRAME	setdiskspare;
		IOCTL_DISK_GET_SPARE_FRAME	getdiskspare;
		IOCTL_GET_SATA_PARAMETER_FRAME	getsataparm;
		IOCTL_SET_SATA_PARAMETER_FRAME	setsataparm;
		IOCTL_GET_POWER_STATUS_FRAME	getpowerstat;
		IOCTL_SET_POWER_STATUS_FRAME	setpowerstat;
		IOCTL_DISK_WIPE_SECTOR_FRAME	diskwipesect;
		IOCTL_GETQDEPTH_FRAME		getqdepth;
		IOCTL_SETQDEPTH_FRAME		setqdepth;
	} payloadstyle;
} IOC_PKG_PAYLOADSTYLE, *PIOC_PKG_PAYLOADSTYLE;

/*******************Payload Style IOCTLs****END********************/



/*******************Change to PASS THROUGH**BEGIN******************/

/* Does it need now?? */
typedef struct _DISK_SET_POWER_STATUS_FRAME_IN {
		USHORT usOPCode;
		BYTE   DevID;
		BYTE   Status;          /* Status: 0 = set spin down, 1 = set spin up */
} DISK_SET_POWER_STATUS_FRAME_IN;
#define MSG_OPCODE_DISK_SET_POWER_STATUS     0x0607
typedef union {
	DISK_SET_POWER_STATUS_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_DISK_SET_POWER_STATUS_FRAME, *PIOCTL_DISK_SET_POWER_STATUS_FRAME;

typedef struct _SET_BOOTABLE_ARY_FRAME_IN {
		USHORT usOPCode;        /* Message OP Code: 0x0C08 */
		BYTE   Index;
		BYTE   Reserved;
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		ULONG  ulReserved[2];   /* Message Unit place holder */
} SET_BOOTABLE_ARY_FRAME_IN;
#define MSG_OPCODE_SET_BOOTABLE_ARY      0x0C08
typedef union {
	SET_BOOTABLE_ARY_FRAME_IN _IN;
	PLACEHOLDER _OUT; /* This is just place holder. For the sake of coding convenience */
} IOCTL_SET_BOOTABLE_ARY_FRAME, *PIOCTL_SET_BOOTABLE_ARY_FRAME;

typedef struct _GET_BOOTABLE_ARY_FRAME_IN {
		USHORT usOPCode;        /* Message OP Code: 0x0D08 */
		BYTE   Reserved[2];
		ULONG  ulReturnHandle;  /* Message Unit place holder */
		ULONG  ulOutputPointer; /* Message Unit place holder */
		ULONG  ulReserved;      /* Message Unit place holder */
} GET_BOOTABLE_ARY_FRAME_IN;
typedef struct _GET_BOOTABLE_ARY_FRAME_OUT {
		BYTE Index;
		BYTE bReserved[3];      /* Message Unit place holder */
} GET_BOOTABLE_ARY_FRAME_OUT;
#define MSG_OPCODE_GET_BOOTABLE_ARY    0x0D08
typedef union {
	GET_BOOTABLE_ARY_FRAME_IN _IN;
	GET_BOOTABLE_ARY_FRAME_OUT _OUT;
} IOCTL_GET_BOOTABLE_ARY_FRAME, *PIOCTL_GET_BOOTABLE_ARY_FRAME;
/*******************Change to PASS THROUGH**END********************/

#endif    /* _RAID_API_IOCTL_H_ */
