/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
/*
 * Description   : All constants used for Message Unit are defined here
 *		   including message types, commands code of each type message,
 *		   event types, events codes, sg types and so on
 */

#ifndef _CONSTANT_H
#define _CONSTANT_H

#define MAX_NUMBER_OF_SG	32/*edl*/
#define MAX_REQ_MSG_SIZE	288
#define MAX_RSP_MSG_SIZE	32
#define MAX_SG_COUNT		MAX_NUMBER_OF_SG
#define MAX_REQ_MSG_COUNT	32
#define MAX_RSP_MSG_COUNT	32

#define REQ_MSG_SIZE_TYPE0	64
#define REQ_MSG_SIZE_TYPE1	160
#define REQ_MSG_SIZE_TYPE2	288
#define REQ_MSG_SIZE_TYPE3	1024

typedef enum {
    MSG_READ		= 1,
    MSG_WRITE,
    MSG_VERIFY,
    MSG_GET_CAPACITY,
    MSG_FLUSH,
    MSG_IOCTL,
    MSG_DISK,
    MSG_ARRAY,
    MSG_MISC,
    /*MSG_ENGINE,*/
    /*MSG_CACHE,*/
    MSG_PASS_THROUGH	= 0x80
}CMD_TYPE;

typedef enum{
    MSG_IOCTL_GET_ADAPTOR_ID,
    MSG_IOCTL_SET_ADAPTOR_ID,
    MSG_IOCTL_GET_ADAPTOR_NAME,
    MSG_IOCTL_SET_ADAPTOR_NAME,
    MSG_IOCTL_ADAPTOR_NUMBER_OF_CHANNEL,
    MSG_IOCTL_SCAN_FOR_CHANNEL,
    MSG_IOCTL_CHANNEL_NUMBER_OF_DEVICE,
    MSG_IOCTL_SCAN_FOR_DEVICE,
    MSG_IOCTL_GET_SMART_SUPPORT_FLAG,
    MSG_IOCTL_SET_SMART_SUPPORT_FLAG,
    MSG_IOCTL_GET_DISK_WRITE_CACHE,
    MSG_IOCTL_SET_DISK_WRITE_CACHE,
    MSG_IOCTL_EVENT_LOG_CHECK,
    MSG_IOCTL_EVENT_LOG_READ,
    MSG_IOCTL_EVENT_LOG_CLEAR,
    MSG_IOCTL_BUZZER_ON,
    MSG_IOCTL_BUZZER_OFF,
    MSG_IOCTL_BUZZER_STATUS,
    MSG_IOCTL_GET_ENCLOSURE_TYPE,
/*    MSG_IOCTL_GET_ENCLOSURE_TEMPERATURE,*/ /* changed to "MSG_IOCTL_GET_ENCLOSURE_STATUS" by David Ting @ 06/12/2002 */
    MSG_IOCTL_GET_ENCLOSURE_STATUS,
    MSG_IOCTL_GET_ENCLOSURE_FAN_SPEED,	/* Removed, no useful longer */
    MSG_IOCTL_GET_ENCLOSURE_33_VOLT,	/* Removed, no useful longer */
    MSG_IOCTL_GET_ENCLOSURE_5_VOLT,	/* Removed, no useful longer */
    MSG_IOCTL_GET_ENCLOSURE_12_VOLT,	/* Removed, no useful longer */
    MSG_IOCTL_BATTERY_STATUS,
    MSG_IOCTL_SET_LED_RED,	/* Removed, no useful longer */
    MSG_IOCTL_SET_LED_ORANGE,	/* Removed, no useful longer */
    MSG_IOCTL_SET_LED_GREEN,	/* Removed, no useful longer */
    MSG_IOCTL_SET_LED_BLACK,	/* Removed, no useful longer */
    MSG_IOCTL_RAM_SIZE,
    MSG_IOCTL_SET_POWER_STATE,
    MSG_IOCTL_GET_POWER_STATE
}MSG_IOCTL_CMD;

typedef enum{
    MSG_DISK_DESCRIPTION,
    MSG_DISK_LOCATION,
    MSG_DISK_GET_SPARE,
    MSG_DISK_READ,
    MSG_DISK_WRITE
}MSG_DISK_CMD;

typedef enum{
    MSG_ARRAY_DESCRIPTION,
    MSG_ARRAY_CREATE,
    MSG_ARRAY_DELETE,
    MSG_ARRAY_MIGRATE,
    MSG_ARRAY_MAINTENANCE_STOP,		/* Added by David Ting @ 05/12/2002 */
    MSG_ARRAY_MAINTENANCE_LOOKUP,	/* Added by David Ting @ 05/12/2002 */
    MSG_ARRAY_INTEGRITY_CHECK,
    MSG_ARRAY_REBUILD,
    MSG_ARRAY_GET_NAME,
    MSG_ARRAY_SET_NAME,
    MSG_ARRAY_SET_REBUILD_RATE,
    MSG_ARRAY_GET_REBUILD_RATE,
    MSG_ARRAY_SET_BOOTABLE,
    MSG_ARRAY_GET_BOOTABLE,
    MSG_ARRAY_RESET
}MSG_ARRAY_CMD;

typedef enum{
    MSG_ENG_GET_THROTTLE_STATUS,/* Removed, no useful longer */
    MSG_ENG_SET_THROTTLE_STATUS	/* Removed, no useful longer */
}MSG_ENG_CMD;

typedef enum{
    MSG_CACHE_GET_WRITE_POLICY,
    MSG_CACHE_SET_WRITE_POLICY,
    MSG_CACHE_GET_FLUSH_FREQUENCY,
    MSG_CACHE_SET_FLUSH_FREQUENCY
}MSG_CACHE_CMD;

typedef enum{
    MSG_MISC_FW_DOWNLOAD,
    MSG_MISC_FW_UPLOAD,
    /*MSG_MISC_ACQUIRE_ADAPTOR_LOCK,*/
    /*MSG_MISC_RELEASE_ADAPTOR_LOCK,*/
    MSG_MISC_GET_FW_VERSION,
    MSG_MISC_ALLOCATE_MEMORY,
    MSG_MISC_GET_TIME,		/* Added by David Ting @ 05/12/2002 */
    MSG_MISC_SET_TIME,		/* Added by David Ting @ 05/12/2002 */
    MSG_MISC_GET_CHANNEL,	/* Added by David Ting @ 06/12/2002 */
    MSG_MISC_RELEASE_CHANNEL	/* Added by David Ting @ 06/12/2002 */
}MSG_MISC_CMD;

typedef enum{
    _32BIT_ADDR,
    _64BIT_ADDR
}SG_TYPE;

typedef enum{
    EVENT_ERROR			= 1,
    EVENT_WARNING,
    EVENT_INFORMATION
}EVENT_TYPE;

typedef enum{
    EVENT_DISK_SET_DOWN		= 1,
    EVENT_DISK_PLUGIN,
    EVENT_RESERVED_SECTOR_ERROR,
    
    EVENT_ARRAY_BAD_SECTOR,
    EVENT_ARRAY_REBUILD,
    EVENT_ARRAY_AUTO_REBUILD,
    EVENT_ARRAY_REBUILD_STOP,
    EVENT_ARRAY_REBUILD_ABORT,
    EVENT_ARRAY_SYNCHRONIZE,
    EVENT_ARRAY_SYNCHRONIZE_STOP,
    EVENT_ARRAY_SYNCHRONIZE_ABORT,
    EVENT_ARRAY_RETRY_TASK,
    EVENT_ARRAY_CREATE,
    EVENT_ARRAY_DELETE,
    EVENT_ARRAY_DOWN,
    EVENT_ARRAY_CRITICAL,
    EVENT_ARRAY_REBUILD_COMPLETE,
    EVENT_ARRAY_SYNCHRONIZE_COMPLETE,
    EVENT_ARRAY_DISK_ERROR,

    EVENT_ENCLOSURE_TEMP_WARNING,
    EVENT_ENCLOSURE_FAN_WARNING,
    EVENT_ENCLOSURE_33V_WARNING,
    EVENT_ENCLOSURE_5V_WARNING,
    EVENT_ENCLOSURE_12V_WARNING
}EVENT_CODE;

typedef enum{
    MU_DEV_DIRECT_ACCESS,
    MU_DEV_SEQ_ACCESS,
    MU_DEV_PRINTER,
    MU_DEV_PROCESSOR,
    MU_DEV_WRITE_ONCE,
    MU_DEV_CDROM,
    MU_DEV_SCANNER,
    MU_DEV_OPTICAL_MEMORY,
    MU_DEV_MEDIUM_CHANGER,
    MU_DEV_COMMUNICATION,

    MU_DEV_ARRAY_CONTROLLER	= 0x0C,
    MU_DEV_ENCLOSURE_SERVICE,
    MU_DEV_REDUCED_BLOCK_COMMAND,
    MU_DEV_OPTICAL_CARD_RW,

    MU_DEV_UNKNOWN		= 0x1F
}MU_DEVICE_TYPE;

typedef enum{
    MU_ENCLOSURE_NONE,
    MU_ENCLOSURE_NO_DRIVE,
    MU_ENCLOSURE_FAST_SWAP,
    MU_ENCLOSURE_SUPPER_SWAP,
    MU_ENCLOSURE_UNIVERSAL_SWAP,
    MU_ENCLOSURE_UNKNOWN	= 0xFFFFFFF
}MU_ENCLOSURE_TYPE;

typedef enum{
    MU_ARRAY_OK,
    MU_ARRAY_WARNING,
    MU_ARRAY_ERROR,

    MU_ARRAY_SYNCHRONIZING	= 0x100,
    MU_ARRAY_REBUIDING		= 0x200,
    MU_ARRAY_MIGRATING		= 0x800,
    MU_ARRAY_MIGRATING_PAUSE	= 0x1000,
    MU_ARRAY_SYSCHRONIZING_PAUSE= 0x2000
}MU_ARRAY_STATUS_MASK;

typedef enum{
    MU_DISK_OK,
    MU_DISK_WARNING,
    MU_DISK_ERROR,

    MU_DISK_FREE		= 0x100,
    MU_DISK_ASSIGNED		= 0x200,
    MU_DISK_SPARE		= 0x400,
}MU_DISK_STATUS_MASK;

typedef enum _MU_CARMEL_DEVICE_STATE{
    MU_CARMEL_D0,
    MU_CARMEL_D1,
    MU_CARMEL_D2,/* Not supported now */
    MU_CARMEL_D3HOT,
    MU_CARMEL_D3COLD	= 0xFFFFFFFF
}MU_CARMEL_DEVICE_STATE;

typedef enum _CARMEL_WORKING_MODE{
    CARMEL_RAID_MODE,
    CARMEL_BRIDGE_MODE
}CARMEL_WORKING_MODE;

typedef enum{
    MU_CACHE_WRITE_BACK,
    MU_CACHE_WRITE_THROUGH
}MU_CACHE_WRITE_POLICY;

typedef enum{
    REQ_MSG_OK                   = 1,
    REQ_MSG_FAILEDINPROC         = 2,
    REQ_MSG_INVALID_TYPE	 = 3,
    REQ_MSG_INVALID_SUBCMD	 = 4,
    REQ_MSG_INVALID_MSG_DATA     = 5,
    REQ_MSG_CARMEL_BUSY          = 6,
    REQ_MSG_SATA_ABORT           = 7,
    REQ_MSG_SATA_MEDIA_ERROR     = 8,
    REQ_MSG_SATA_FATAL_ERROR     = 9,
    REQ_MSG_SATA_DISK_ERROR      = 10,
    REQ_MSG_CMD_ABORT            = 11   
}REQ_MSG_ERROR_CODE;

typedef enum {
	SPEC_RESP_EVENT		= 0x01,
	SPEC_RESP_XTASK		= 0x02,
	SPEC_RESP_ARRSTAT	= 0x04,
	SPEC_RESP_HOTPLUG	= 0x05,
	SPEC_RESP_INTERR	= 0x06,
	SPEC_RESP_SMART		= 0x07,
	SPEC_RESP_ARRCHANG	= 0x08
} SPEC_RESP_TYPE;

/* Following for Smart */

#define IDE_STATUS_MASK	0xE9

/* Pass Through Cmd Protocol definition */
#define NONE_DATA		0x01
#define PIO_DATA_IN		0x02
#define PIO_DATA_OUT		0x03

#define WIN_SMART		0xB0 /* self-monitoring and reporting */

/* Password used in TF4 & TF5 executing SMART commands */

#define SMART_LCYL_PASS			0x4F
#define SMART_HCYL_PASS			0xC2

#endif
