/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
/*
 * Description   : safte.h - The codes to achieve saf-te
 */

#ifndef _SAFTE_HEAD
#define _SAFTE_HEAD

#if !defined(U8)
#define U8	unsigned char
#endif

#if !defined(U16)
#define	U16	unsigned short
#endif

#if !defined(U32)
#define U32	unsigned long
#endif

#define	MAXCHANNEL	8		/* Carmel has 8 sata channel */
#define	MAXDEVICE	MAXCHANNEL	/* each sata channel can only connect one sata device */

/*
 * The following are BuffId value list
 */
#define	GET_ENC_CONF	0x00
#define	GET_ENC_STAT	0x01
#define	GET_ENC_USAGE	0x02
#define	GET_DEV_INSERT	0x03
#define	GET_DEV_STAT	0x04


/*********************************************************************
 *			The following are definition of return data              *
 *********************************************************************/

/*
 * data of Enclosure Configuration (00h)
 */
typedef struct _Enclosure_Conf {
	U8	fan_num;		/* number of fans (f) */
	U8	powsup_num;		/* number of power supplies (p) */
	U8	devslot_num;		/* number of device slots (d) */
	U8	doorlock;		/* door lock installed */
	U8	tempsen_num;		/* number of temperature sensors (t) */
	U8	speaker;		/* speakers installed */
	U8	reserved[57];		/* reserved 6..62 */
	U8	vendspec_num;		/* number of vendor specific bytes (v) */
#if 0
	U8 	vendor[0];		/* Vendor Specific */
#endif
} Enclosure_Conf;


/*
 * Data of Enclosure status (01h)
 */

/*
 * Fan Status fields - These fields will contain one of the following values
 */
#define	FAN_OP		0x00		/* Fan is operational.	*/
#define	FAN_MALFUN	0x01		/* Fan is malfunctioning. */
#define	FAN_NOINST	0x02		/* Fan is not installed. */
#define	FAN_UNKNOW	0x80		/* Unknown Status, or Status Not Reportable */

/*
 * Power Supply Status fields - These fields will contain one of the following values 
 */
#define	POW_OPON	0x00		/*	Power supply is operational and on */
#define	POW_OPOFF	0x01		/* 	Power supply is operational and off */
#define	POW_MALON	0x10		/* 	Power supply is malfunctioning and commanded on */
#define	POW_MALOFF	0x11		/*  Power supply is malfunctioning and commanded off */
#define	POW_NOEXIST	0x20		/* 	Power supply is not present */
#define POW_EXIST	0x21		/* 	Power supply is present 
					 *  (This status may be used instead of codes 00h - 11h if 
					 *  failure detection and On/Off status are not detectable)
					 */
#define	POW_UNKNOWN	0x80		/* 	Unknown Status, or Status Not Reportable */

/* definitions for Power State */
#define	PowerOFF		POW_OPOFF
#define	PowerON			POW_OPON


/*
 * Door Lock Status - This field will contain one of the following values
 */
#define	DOOR_LOCKED	0x00		/* Door is currently locked */
#define DOOR_UNLOCK	0x01		/* Door is currently unlocked, or no SEP controllable 
					 * door lock installed */
#define	DOOR_UNKNOW	0x80		/* Unknown Status, or Status Not Reportable */

/*
 * Speaker Status - This field will contain one of the following values
 */
#define	SP_OFF	0x00			/* Speaker is currently off, or no speaker installed */
#define	SP_ON	0x01			/* Speaker is currently on */

/*
 * Temperature - Each of these fields contains an unsigned integer
 * indicating internal enclosure temperature at this sensor in degrees Fahrenheit.
 * This value, from 0 to 255, is based on a scale starting at -10 degrees.  
 * This gives an effective reportable temperature range from -10 degrees to 245 degrees.
 */
/* temperature range from -10 degrees to 245 degrees. */
#define	TEMP_MIN	-10
#define	TEMP_MAX	245


/*
 * Temperature Out Of Range Flags
 * These fields contain 16 flags which indicate whether or 
 * not the SEP device has detected a temperature that is 
 * out of normal operating range.  Fifteen flags are provided 
 * for individual sensors, and one flag is provided for an overall temperature alert.
 */
typedef struct _TEMP_OUTRANGE_FLAG1 {
	U8	t8:1,t9:1,t10:1,t11:1,t12:1,t13:1,t14:1;
	U8	eta:1;		/* whether there is an enclosure temperature alert 
				 * on any sensor within the enclosure 
				 */
} TEMP_OUTRANGE_FLAG1;

typedef struct _TEMP_OUTRANGE_FLAG2 {
	U8	t0:1,t1:1,t2:1,t3:1,t4:1,t5:1,t6:1,t7:1;
} TEMP_OUTRANGE_FLAG2;

/*
 * Enclosure status data type
 */
typedef struct _Enclosure_Stat {
	U8	FanStat[MAXDEVICE];		/* Fan Status fields */
	U8	PowerStat[MAXDEVICE];		/* Power Supply Status fields */
	U8	SmbusIdMapChan[MAXDEVICE];	/* SMBUS Device ID field */
	U8	DoorLock;			/* Door Lock Status */
	U8	Spearker;			/* Speaker Status */
	U8	Temperature[MAXDEVICE];		/* Temperature field */
	TEMP_OUTRANGE_FLAG1	TempFlag1;	/* Temperature Out Of Range Flags 1 */
	TEMP_OUTRANGE_FLAG2	TempFlag2;	/* Temperature Out Of Range Flags 2 */
	U8	NumVendSpec;			/* Number of Vendor Specific Bytes */
#if 0
	U8	VendSpec[0];			/* Vendor Specific */
#endif
} Enclosure_Stat;



/*
 * Read Usage Statistics (02h)
 */
typedef struct _Enclosure_UsageStat {
	char MSBPowerOnMin;
	char MID0PowerOnMin;
	char MID1PowerOnMin;
	char LSBPowerOnMin;
	char MSBPowerOnCyc;
	char MID0PowerOnCyc;
	char MID1PowerOnCyc;
	char LSBPowerOnCyc;
	char reserved[7];
	char vendspec_num;
#if 0
	char vendspec[0];
#endif
} Enclosure_UsageStat;



/*
 * Read Device Insertions (03h)
 */
typedef struct _Device_Insertions {
	char MSBNumSlot0;		/* first byte of Devince insertion number on channel 0 */
	char LSBNumSlot0;		/* second byte of device insertion number on channel 0 */
	char MSBNumSlot1;		/* first byte of Devince insertion number on channel 1 */
	char LSBNumSlot1;		/* second byte of device insertion number on channel 1 */
	char MSBNumSlot2;		/* first byte of Devince insertion number on channel 2 */
	char LSBNumSlot2;		/* second byte of device insertion number on channel 2 */
	char MSBNumSlot3;		/* first byte of Devince insertion number on channel 3 */
	char LSBNumSlot3;		/* second byte of device insertion number on channel 3 */
	char MSBNumSlot4;		/* first byte of Devince insertion number on channel 4 */
	char LSBNumSlot4;		/* second byte of device insertion number on channel 4 */
	char MSBNumSlot5;		/* first byte of Devince insertion number on channel 5 */
	char LSBNumSlot5;		/* second byte of device insertion number on channel 5 */
	char MSBNumSlot6;		/* first byte of Devince insertion number on channel 6 */
	char LSBNumSlot6;		/* second byte of device insertion number on channel 6 */
	char MSBNumSlot7;		/* first byte of Devince insertion number on channel 7 */
	char LSBNumSlot7;		/* second byte of device insertion number on channel 7 */
} Device_Insertions;


/*
 * Device Status
 */
typedef struct _DeviceStat {
	U8 noerror:1;			/* Bit0: No Error Flag.  Set if there are no outstanding */
					/* error conditions on this device						 */

	U8 devfault:1;			/* Bit1: Device Faulty Flag.  Set if this device has     */
					/* exhibited some hardware or data fault				 */
	
	U8 devrebuild:1;		/* Bit2: Device Rebuilding Flag.  Set if the device is   */
					/* being rebuilt.                                        */
	
	U8 InFailedArray:1;		/* Bit3: In Failed Array Flag.  Set if this device is in */
					/* an array with an error on any member device.          */
	
	U8 InCritArray:1;		/* Bit4: In Critical Array Flag.  Set if this device is  */
					/* in an array that is non fault-tolerant.               */
	
	U8 ParityCheck:1;		/* Bit5: Parity Check Flag.  Set if the device is in an  */
					/* array which is undergoing a parity check operation    */
	
	U8 PredictedFault:1;		/* Bit6: Predicted Fault Flag.  Set if the drive has been */
					/* tagged by a fault prediction algorithm as being likely */
					/* to fail in the near future.                            */
	
	U8 NoDrive:1;			/* Bit7: No Drive Flag.  Set if this slot has no drive inserted. */
} DeviceStat_t;

/*
 * Read Device Status (04h)
 */

/*
 * One Device status data struct 
 */
typedef struct _GetDeviceStat1 {
	DeviceStat_t	sds;		/* a copy of the three bytes of flags most recently sent */
					/* for the respective drive/slot using the Write Device  */
					/* Status command                                        */
	U8 	reservedbytes[2];	/* Byte 1,2 are reserved */
	U8	DevInserted:1;		/* Bit0: Device Inserted Flag - Indicates whether or not     */
					/* there is a physical device inserted in this slot.         */
					/* This does not imply that the device is ready for access.  */
					/* 0 - no device inserted,  1 - device inserted.			 */
	U8	ReadyInsRev:1;		/* Bit1: Ready for Insertion/Removal Flag - This flag        */
					/* indicates whether or not the slot is ready for the physical*/
					/* insertion or removal of a device.  0 - not ready,  1 - ready. */
	U8	PreparedOp:1;		/* Bit2: Prepared For Operation Flag - This flag indicates    */
					/* whether or not the slot is activated so that the inserted  */
					/* drive may be accessed.  0 - not activated,  1 - activated. */
	U8	reserved:5;		/* bits 3-7 are reserved  */
} GetDeviceStat1;


/*********************************************************************
 *			The following are definition of Write data              *
 *********************************************************************/

/*
 *	The following are write command list
 */
#define	SET_DEV_STAT	0x10
#define	SET_BUS_ID	0x11
#define	SET_DEV_OP	0x12
#define	SET_FAN_SPEED	0x13
#define	SET_POWER_STAT	0x14
#define	SET_GLOB_CMD	0x15

/*
 * Write Device Status (10h)
 */

/*
 * Set One Device Status Data Structure
 */
typedef struct _SetDeviceStat1 {
	DeviceStat_t	sds;		/* State of each of it's associated slots and */
					/* the devices potentially inserted			  */
	U8 	reserved[2];		/* Byte 1,2 are reserved */
#if 0
	U8	VendSpec[0];		/* Vendor specific fields may be implemented, */
					/* but will be ignored by a target SEP device */
					/* that is not aware of them                  */
#endif
} SetDeviceStat1;


/*
 * Set Bus ID (11h)
 */
typedef struct _SetBusId {
	U8	OpCode;			/* Operation Code - Must be 11h for the Set Bus ID command.   */
	U8	ChannelNo;		/* Device Slot Number (channel No.)- The slot number  */
					/* of the device on which to set the Bus ID.          */
	U8	BusID;			/* Bus ID - This field holds the SMBus ID that should be set. */
	U8	reserved[61];		/* Reserved from 3 - 63 bytes */
#if 0
	U8	VendSpec[0];		/* Vendor Specific            */
#endif
} SetBusId;



/*
 * Perform Slot Operation (12h)
 */
typedef struct _SetSlotOp {
	U8	OpCode;			/* Operation Code - Must be 12h for the Perform Slot Operation command. */
	U8	ChannelNo;		/* Slot Number (Channel No.) - This field indicates which drive */
					/* slot to prepare to perform the given operation on.           */
	U8	reservedbits:5;		/* reserved bits 3-7  of op flag */
	U8	Identify:1;		/* Identify Flag. - This flag indicates that the SEP device     */
					/* should indicate the physical location of this slot in some   */
					/* manner, to indicate to the operator which slot is being      */
					/* addressed.  This is usually done by flashing an adjacent LED */
					/* in a unique pattern                                          */
	U8	PrepInsRev:1;		/* Prepare For Insertion Or Removal Flag. - This bit indicates  */
					/* that the SEP device should take any action required to allow */
					/* the safe physical insertion or removal of a device from this slot.*/
	U8	PrepOp:1;		/* Prepare For Operation Flag. - This flag indicates that the SEP */
					/* device should take any action required to make the device in   */
					/* this slot available on the bus.                                */
	U8	reservedbytes[61];	/* Reserved Bytes from 3 - 63 */
#if 0
	U8	VendSpec[0];		/* Vendor Specific            */
#endif
} SetSlotOp;


/*
 * Set Fan Speed (13h)
 */
typedef struct _SetFanSpeed {
	U8	Opcode;			/* Operation Code - Must be 13h for the Set Fan Speed command. */
	U8	FanNum;			/* Fan Number - Indicates which fan on which to set the speed.  */
					/* This can range from 0 to f-1.                                */
	U8	SpeedForFan;		/* Speed for Fan  - This fields holds the value for the desired */
					/* speed setting.  It should hold one of the following values - */
					/*	0 - turn fan off
					 *	1 - run fan at quarter capacity
					 *	2 - run fan at half capacity
					 *	3 - run fan at three quarters capacity
					 *	4 - run fan at full capacity
					 */
	U8	Reserved[13];
} SetFanSpeed;



/*
 * Activate Power Supply (14h)
 */
typedef struct _SetPowerSupply {
	U8	OpCode;			/* Operation Code - Must be 14h for the Activate Power Supply command.   */
	U8	PowSupNum;		/* Power Supply Number - Indicates which power supply to turn on */
					/* or off.  This can range from 0 to p-1                         */
	U8	PowerStat;		/* Power State - This field should hold one of the following     */
					/* values -                                                      */
					/*		00h - turn power supply off
					 *		01h - turn power supply on
					 */
	U8	Reserved[13];		/* Reserved bytes 3 - 15 */
} SetPowerSupply;

typedef	struct _GlobalStat {
	U8	AudibleAlarm:1;		/* bit0: Audible Alarm Control.  Set to sound an audible alarm   */
					/* within the enclosure.  Reset to turn off the alarm.           */ 
	U8	GlobalFail:1;		/* bit1: Global Failure Indication.  Set to request that the     */
					/* enclosure perform some actions (alarm, LEDs, LCD message)     */
					/* that indicate a global error condition.                       */
	U8	GlobalWarn:1;		/* bit2: Global Warning Indication.  Set to request that the     */
					/* enclosure perform some actions (alarm, LEDs, LCD message)     */
					/* that indicate a global warning condition.                     */
	U8	EnclosurePower:1;	/* bit3: Enclosure Power.  Set to turn off power to the enclosure. */
					/* Clear to turn on power to the enclosure                       */
	U8	CoolFail:1;		/* bit4: Cooling Failure.  Set to indicate that a fan/cooling    */
					/* failure has occurred */
	U8	PowerFail:1;		/* bit5: Power Failure.  Set to indicate that system power has   */
					/* been lost. This allows the enclosure to perform any emergency */
					/* shutdown procedure required before battery power is lost.     */
	U8	DrvFail:1;		/* bit6: Drive Failure.  Set to drive any global enclosure       */
					/* indicator that a drive has failed                             */
	U8	DrvWarn:1;		/* bit7: Drive Warning.  Set to drive any global enclosure       */
					/* indicator that a potential error has or may have occurred on  */
					/* a drive.  */
	U8	ArrayFail:1;		/* Bit0: Array failure.  Set to drive any global enclosure       */
					/* indicator that an array has failed                            */
	U8	ArrayWarn:1;		/* Bit1: Array Warning.  Set to drive any global enclosure       */
					/* indicator that an array is non fault-tolerant.                */
	U8	EnclosureLock:1;	/* Bit2: Enclosure Lock.  Set to lock the enclosure drive bay    */
					/* access door.  Cleared to unlock the door.                     */
	U8	Reservedbits:5;		/* Reserved bits 3-7 in flag byte 2 */

} GlobalStat_t;

/*
 * Send Global Command (15h)
 */
typedef struct _SetGlobalcmd {
	U8	OpCode;			/* Operation Code - Must be 15h for Send Global Command. */
	GlobalStat_t gs;		/* global status */
	U8	FlagByte3;		/* Reserved flag byte 3 */
	U8	Reserved[12];		/* Reserved bytes 4 - 15 */
} SetGlobalcmd;

/* 8 bits + 3 bits */
#define	MAXGLOBCMDBIT	11

/******************************************************************************************
 *               The following are Data struct for Firmware SMBUS Interface               *
 ******************************************************************************************/

#if 0
/* 
 * Map sata channel to smbus id
 */
typedef struct _ChanMapSmbus[MAXCHANNEL] ChanMapSmbus;
#endif

typedef struct _HdAryStat {
	DeviceStat_t	ds;		/* Drive status */
	U8		aryid;		/* The array id this driver is assigned to */
} HdAryStat_t;

/*
 * Save each enclosure power status
 */
typedef struct _EncPowerStat {
	U8 Enc0Power:1;			/* Bit0: 0 enclosure0 power down;
						1 enclosure0 power up */
	U8 Enc1Power:1;			/* Bit1: 0 enclosure1 power down;
						1 enclosure1 power up */
	U8 Enc2Power:1;			/* Bit2: 0 enclosure2 power down;
						1 enclosure2 power up */
	U8 Enc3Power:1;			/* Bit3: 0 enclosure3 power down;
						1 enclosure3 power up */
	U8 Enc4Power:1;			/* Bit4: 0 enclosure4 power down;
						1 enclosure4 power up */
	U8 Enc5Power:1;			/* Bit5: 0 enclosure5 power down;
						1 enclosure5 power up */
	U8 Enc6Power:1;			/* Bit6: 0 enclosure6 power down;
						1 enclosure6 power up */
	U8 Enc7Power:1;			/* Bit7: 0 enclosure7 power down;
						1 enclosure7 power up */
} EncPowerStat;

typedef struct _BoxStat {
	U8	boxsum;			/* how many enclosure exist actually */
	EncPowerStat ep;		/* Each channel enclosure's power */
	U8	workledinited;		/* the work leds have been inited? */
	U8	statledinited;		/* the status leds have been inited? */
	U8	BoxType[MAXCHANNEL];	/* Save each channel box type */
	HdAryStat_t has[MAXCHANNEL]; 	/* save each device slot status written last time */
					/* the value are also modified when Led is set!!! */
	U8 	ArrayStatLed[MAXCHANNEL]; /* Each array status led: 
						0x00 normal; 0x01 warning; 0x10: error */
	U8	ArrayActiveLed[MAXCHANNEL]; /* Each array avtive status led */
	U8	ChIdmapSbId[MAXCHANNEL];/* Channel map to SMBus ID */
	U32	FanSpeed1[MAXCHANNEL];
	U32	FanSpeed2[MAXCHANNEL];
	U32	temp1[MAXCHANNEL];
	U32	temp2[MAXCHANNEL];
} BoxStat_t;

typedef struct _Safte_Status {
	U8	fan_num;		/* number of fans (f) */
	U8	powsup_num;		/* number of power supplies (p) */
	U8	devslot_num;		/* number of device slots (d) */
	U8	tempsen_num;		/* number of temperature sensors (t) */
	U8	doorlock;		/* door lock installed */
	U8	speaker;		/* speakers installed */
	GlobalStat_t gs;		/* global status */
} Safte_Status;



/* Sfate context: 16 bytes */
#define SAFTE_CONTEXT_SIZE 16

typedef struct _Safte_Context {
	u8		*regbaddr;	/* the base address of smbus registers: 4 bytes */
	Safte_Status	sstat;		/* Safte status: 12 bytes */
	BoxStat_t	lstat;		/* Enclosure Led status */
} Safte_Context_t, *pSafte_Context_t;


/*
 * The maximum channel number
 */
#define	MAX_SATA_CHNL	8

/*
 * The Maximum number of Cards we will support
 */
#define	MAX_CARDS	4


/*
 * Array status 
 */
#if !defined(ARRAY_NORMAL)
#define	ARRAY_NORMAL	00 
#endif

#if !defined(ARRAY_WARNING)
#define	ARRAY_WARNING	01
#endif

#if !defined(ARRAY_ERROR)
#define	ARRAY_ERROR	10
#endif

extern pSafte_Context_t pSafteContext[MAX_CARDS];

#endif

