/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */

/*
 * Description   : safte.h - The codes to achieve saf-te
 */

#if !defined(__SAFTE_H_)
#define	__SAFTE_H_

#if !defined(U32)
#define	U32 unsigned int
#endif


typedef struct _HdAryInfo {
	unsigned char has;	/* HD status */
	unsigned char aryid;	/* the array id that the HD is assigned to */
} HdAryInfo_t;

/* the following are for HD status */
#define	NOERROR		0x01
#define	DEVFAULT	0x02
#define	DEVREBUILD	0x04
#define	INFAILEDARRAY	0x08
#define	INCRITARRAY	0x10
#define	PARITYCHECK	0x20
#define	PREDICTEDFAULT	0x40
#define	NODRIVE		0x80

/* Following are for aryid */
#define	FREEDISK	0xff
#define FIRSTARY	0x0
#define	MAXARYNO	0x7

/* extern function to get each channel status from firmware by OSD driver */
/* Detail: 
 *	Input:
 *	int	maxchanum:	maximum channel exist on carmel (MAXCHANNEL 8/4 usually)
 *	HdAryInfo_t *phai:  a pointer to a HdAryInfo_t array that will save HD and array
 * 			    status, the element number of the array is the channel no.
 *	int cardno:         The card no ( for support 4 cards )
 *
 *	Outpu:
 *		return a > 0 value that how many channels have hard disk.
 *			 = 0 means that isn't any hard disk on any channel.
 *			 < 0 means the function op failed
 */				
extern int  getChStatusFromFW(int maxchnum, HdAryInfo_t *phai, int cardno);

/* Interface function of Safte module */



/*
 *
 * The function description:
 *	This function initialize safte module context data, and do some init
 * action on smbus
 *
 * Input parameters:
 *	U32 baddr:	SMBus register base address
 *	int cardno:	card no (for support 4 cards )
 *
 * Output:
 *	If success, pSafteContext will point to the address of buffer holding 
 * safte context, smbus handle will be set to route to firmware,  and smbus will 
 * be initialized, and return value 0
 *   	If failed, psc will be null, and return value will be -1
 *
 */
extern int InitSafte(U8 *, int);


/*
 *
 * The function description:
 *		This function do all Safte module's actions
 *
 * Input Parameters:
 *	 U32 cmd:  SCSI SAFTE R/W command that should be
 *				GET_ENC_CONF,	GET_ENC_STAT,
 *				GET_ENC_USAGE,	GET_DEV_INSERT,
 *				GET_DEV_STAT,
 *				SET_DEV_STAT,	SET_BUS_ID,
 *				SET_DEV_OP,	SET_FAN_SPEED,
 *				SET_POWER_STAT,	SET_GLOB_CMD
 *
 *	 void *data:	The input data if cmd is write command
 *					else the buffer address to hold the return data
 *
 *	int cardno:	card's no (for support 4 cards)
 *
 * Output:
 *	  If this function success, return value 0, and 
 *	  pSafte_Context_t psc will hold some safte module's status if cmd is 
 *        write cmd. if cmd is read command, void *data will hold the return
 *        data
 *	  If this function failed, return value -1
 * 	  this function whould use >64 byte stack space.
 *
 */
extern int submitSafte(U32, void *, int);

#if !defined(ARRAY_NORMAL)
#define	ARRAY_NORMAL	00 
#endif

#if !defined(ARRAY_WARNING)
#define	ARRAY_WARNING	01
#endif

#if !defined(ARRAY_ERROR)
#define	ARRAY_ERROR	10
#endif

/*
 * The function is used to set drive status led of enclosure on each channel
 * as status of array the drive be assigned to
 *
 * Input:
 *	int aryid:	The array Id present which array is processed
 *
 *	int ArrayStat:
 *		Array status: ARRAY_NORMAL(0x00) for normal, 
 *			      ARRAY_WARNING(0x01) for warning,
 *			      ARRAY_ERROR(0x10) for error
 *	
 *	int HDnum:	How manay hard disk the array has
 *
 *	U32 HDList:
 *		It is an 32-bit mapping, each 4 bits present each channel; 
 *		bits [4n, 4n+1, 4n+2,4n+3] isn't 0x0e or 0x0f to indicate 
 * 		the corresponding channel is assigned to this array
 *		these [4n,...,4n+3] is 0x0f to indicate the disk on this channel 
 *		isn't assigned to this array if disk exist on this channel or 
 *		no hard disk exist on this channel
 *		[4n...4n+3] is equal to 0x0e means the hard disk is failed or offline
 *
 *	int cardno:
 *		card's no (for support 4 card )
 * Ouput:
 *	If success, return value 0; or return -1;
 *
 */
extern  int SetArrayStatLed(int, int, int, U32, int);
 
/*
 * The function is used to set drive status led of enclosure on each channel
 * as status of array the drive be assigned to
 *
 * Input:
 *	unsigned char HDList:
 *		It is an 8-bit mapping; bit n is set one to indicate 
 * 		the disk on the corresponding channel is accessed, or
 *		means the disk on the channel is idle if exist
 *
 *	int cardno:
 *		card's no (for support 4 card )
 *	
 * Ouput:
 *	If success, return value 0; or return -1;
 *
 */ 
extern int SetDiskWorkLed(unsigned char, int);

/*
 * The function will be called to set the enclosure led to be plugin or 
 * plug-out/error status;
 * The function should be called when "Plug-In" or "Set Disk Down" Event happened
 * And it will call SetArrayStatLed() internal
 *	Input:
 *		char cardno:	The carmel card's no.
 *		char chid:	The No of the channel the disk located at
 *		char status:	Present the disk's status
 *				0 --- Plug-Out/Error;
 *				1 --- Plug-In
 *
 *	Out:
 *		if success, return 0, or return -1
 *
 */
extern int	SetDiskPlugInOutLed(int cardno, char chid, char status);

extern int SetLedInShutdown(int chanid, int cardno);

/* Micro-definition for status */
#define	DISKPLUGOUT	0	/* DISK Plug-Out/Error */
#define	DISKPLUGIN	1	/* DISK Plug-In	*/

unsigned char GetChanMapSmbus(int cardno, int channo);

void SetChIdmapSbId(int cardno, int chan, unsigned char sbid);

#endif	/* __SAFTE_H_ */
