/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for Linux.
 *
 * Copyright (c) 2001  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2001 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
 
/*
 * Description   : smbus.h - The codes to constrol smbus
 */

#ifndef __SMBUS_H__
#define __SMBUS_H__

#define TRACE Print

/* SMBUS Register base address offset from BAR0 */
#define	SMBUS_BADDR	(0x30000)

#define	SMST	(SMBUS_BADDR + 0)	/* Status Register */
#define	SMIS	(SMBUS_BADDR + 1)	/* Interrupt Status Register */
#define	SMIM	(SMBUS_BADDR + 2)	/* Interrupt Mask Register */
#define	SMCO	(SMBUS_BADDR + 3)	/* Control Register */
#define	SMSA	(SMBUS_BADDR + 4)	/* Slave Address Register */
#define	SMDT	(SMBUS_BADDR + 0x10)	/* Data Register */
#define	SMRS	(SMBUS_BADDR + 0x20)	/* Reset Register */

#ifndef bit
#define bit(x) (1<<x)
#endif

/* For Status Register */
#define	SMST_SLAVE		bit(0)
#define	SMST_MASTER		bit(1)
#define	SMST_BUF_EMPTY		bit(2)
#define SMST_SDA		bit(4)
#define SMST_SCL		bit(5)


#define SM_SCL	SMST_SCL
#define	SM_SDA	SMST_SDA

#define SM_ADDRESS_READ 0x01
#define SM_ADDRESS_WRITE 0x00

#if 0
typedef unsigned char  	U8;
typedef unsigned short 	U16;
typedef unsigned long   U32;

typedef U8 *	PU8;
typedef U16 *  	PU16;
typedef U32 * PU32;
#else

#if !defined(U8)
#define	 U8	unsigned char
#endif

#if !defined(UINT8)
#define	UINT8	unsinged char
#endif

#if !defined(U16)
#define	U16	 unsigned short
#endif

#if !defined(UINT16)
#define	UINT16	unsinged short
#endif

#if !defined(U32)
#define	U32	 unsigned long
#endif

#if !defined(UINT32)
#define	UINT32	unsinged int
#endif

#if !defined(PU8)
#define	PU8	 U8 *
#endif

#if !defined(PU16)
#define	PU16	 U16 *
#endif

#if !defined(PU32)
#define	PU32	 U32 *
#endif

#if !defined(PULONG)
#define PULONG	unsigned long *
#endif

#endif	/* 0 */

extern int SMBus_init(U8 *);
extern U16 SMBus_ReadWord(U8, U8, U8 *);
extern void SMBus_WriteWord(U8, U8, U16, U8 *);
extern U8 SMBus_ReadBlock(U8, U8, PU8, U8, U8 *);

extern int SMBus_Simple(U8, U8 *);
extern U8 SMBus_ReadOneByte(U8, U8 *);
extern int SMBus_WriteOneByte(U8, U8, U8 *);
extern U16 SMBus_WinbondReadTwoBytesFromPresetPointerLocation(U8, U8 *);
extern int SMBus_WriteBytes(U8 *, U8 *, int);

#endif /* __SMBUS_H__ */

