/*+M*************************************************************************
 * Promise SATAII S150 SX8 device driver for LINUX.
 *
 * Copyright (c) 2001-2004  Promise Technology, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * --------------------------------------------------------------------------  
 * Copyright (c) 1999-2004 Promise Technology, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer,
 *    without modification, immediately at the beginning of the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Where this Software is combined with software released under the terms of 
 * the GNU Public License ("GPL") and the terms of the GPL would require the 
 * combined work to also be released under the terms of the GPL, the terms
 * and conditions of this License will apply in addition to those of the
 * GPL with the exception of any terms or conditions of this License that
 * conflict with, or are expressly prohibited by, the GPL.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *-M************************************************************************
 */
 
 
/*
 * Description   : Host message format between OSD and Firmware and
 *			The event log format.
 */

#ifndef _TYPE_H
#define _TYPE_H

#include "constant.h"

#ifndef INT8
#define INT8	char
#endif

#ifndef INT16
#define INT16	short
#endif

#ifndef INT32
#define INT32	int
#endif

#ifndef UINT8
#define UINT8	unsigned char
#endif

#ifndef UINT16
#define UINT16	unsigned short
#endif

#ifndef UINT32
#define UINT32	unsigned int
#endif

typedef INT8	*PINT8;
typedef INT16	*PINT16;
typedef INT32	*PINT32;
typedef UINT8	*PUINT8;
typedef UINT16	*PUINT16;
typedef UINT32	*PUINT32;

typedef UINT8 CMD_MSG_TYPE0[REQ_MSG_SIZE_TYPE0];
typedef UINT8 CMD_MSG_TYPE1[REQ_MSG_SIZE_TYPE1];
typedef UINT8 CMD_MSG_TYPE2[REQ_MSG_SIZE_TYPE2];
typedef UINT8 CMD_MSG_TYPE3[REQ_MSG_SIZE_TYPE3];

typedef struct{
    UINT32 addr;
    UINT32 length;
}MSG_SG;



/********************************************************
 *		Response message format			*
 ********************************************************/
/*typedef struct _RESPONSE_MSG{
    UINT32 status;
    UINT32 returnHandle;
}RESPONSE_MSG;*/
typedef struct _RESPONSE_MSG{
    UINT32 returnHandle;
    UINT32 status;
}RESPONSE_MSG;


/********************************************************
 *		Commands message format			*
 ********************************************************/

/**********************************************************
 *	Read, Write, Verify command message format
 */
typedef struct _STRUCT_MSG_RWV{
    union{
	UINT8 cmd;
	UINT8 type;
	UINT8 subCmd;	/* The Range of Value:
			 *	MSG_READ,
			 *	MSG_WRITE,
			 *	MSG_VERIFY
			 */
    } RW;
    UINT8 arrayId;
    UINT8 sgCount;
    UINT8 sgType;
    UINT32 returnHandle;
    UINT32 lba;
    UINT16 blockCount;
    UINT16 lbaHigh;
    struct{
	UINT32 addr;
	UINT32 length;
    }sg[MAX_NUMBER_OF_SG];
    struct{
	UINT32 highAddr;
	UINT32 reserved;
    }SUPPORT_64BITS_ADDR;
}STRUCT_MSG_RWV,STRUCT_MSG_READ,STRUCT_MSG_WRITE,STRUCT_MSG_VERIFY;

/**********************************************************
 *	Get Capacity command message format
 */
typedef struct _STRUCT_MSG_GET_CAPACITY{
    union{
	UINT8 cmd;	/* The Range of Value:
			 *	MSG_GET_CAPACITY
			 */
	UINT8 type;
	UINT8 subCmd;
    } HD;
    UINT8 arrayId;
    UINT8 reserved[2];
    UINT32 returnHandle;
    UINT32 oDataPtr;
    UINT32 reserved1;
}STRUCT_MSG_GET_CAPACITY;

/**********************************************************
 *	Flush command message format
 */
typedef struct _STRUCT_MSG_FLUSH{
    union{
	UINT8 cmd;	/* The Range of Value:
			 *	MSG_FLUSH
			 */
	UINT8 type;
	UINT8 subCmd;
    } HD;	
    UINT8 reserved[3];
    UINT32 returnHandle;
    UINT32 reserved2[2];
}STRUCT_MSG_FLUSH;

/**********************************************************
 *	IO System IOCTLs command message format:
 *		Adaptor Number of Channels
 *		Scan for Channel
 *		Channel Number of Devices
 *		Scan for Devices
 *		Get S.M.A.R.T. Support Flag
 *		Set S.M.A.R.T. Support Flag
 *		Get Disk Write Cache
 *		Set Disk Write Cache
 *		Buzzer Status
 *		Battery Status
 *		RAM Size
 *		Get Power State
 *		Set Power State
 */
typedef struct _STRUCT_MSG_IOCTL{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;	/* The Range of Value:
			 *	MSG_IOCTL
			 */
	    UINT8 subCmd;/* The Range of Value:
			  *	MSG_IOCTL_ADAPTOR_NUMBER_OF_CHANNEL
			  *	MSG_IOCTL_SCAN_FOR_CHANNEL
			  *	MSG_IOCTL_CHANNEL_NUMBER_OF_DEVICE
			  *	MSG_IOCTL_SCAN_FOR_DEVICE
			  *	MSG_IOCTL_GET_SMART_SUPPORT_FLAG
			  *	MSG_IOCTL_SET_SMART_SUPPORT_FLAG
			  *	MSG_IOCTL_GET_DISK_WRITE_CACHE
			  *	MSG_IOCTL_SET_DISK_WRITE_CACHE
			  *	MSG_IOCTL_BUZZER_ON
			  *	MSG_IOCTL_BUZZER_OFF
			  *	MSG_IOCTL_GET_ADAPTOR_ID
			  *	MSG_IOCTL_GET_ENCLOSUER_TYPE
			  *	MSG_IOCTL_GET_ENCLOSUER_STATUS
			  *	MSG_IOCTL_BUZZER_STATUS
			  *	MSG_IOCTL_BATTERY_STATUS
			  *	MSG_IOCTL_RAM_SIZE
			  *	MSG_IOCTL_SET_POWER_STATE
			  *	MSG_IOCTL_GET_POWER_STATE
			  */
	} TP;
    } HD;
    union{
	UINT8 adaptorId;
	UINT8 channelId;
	UINT8 reserved1;
    } ID;
    union{
	UINT8 reserved2;
	UINT8 index;
    } IX;
    UINT32 returnHandle;
    UINT32 oDataPtr;
    union{
	UINT32 reserved3;
	UINT32 smartSupportFlag;/* only for Set S.M.A.R.T. Support Flag */
	UINT32 diskWriteCacheFlag;/* only for Set Disk Write Cache */
	UINT32 state;/* only for Set Power State */
    } FLAG;
}STRUCT_MSG_IOCTL;

typedef struct _STRUCT_MSG_IOCTL_ADAPTOR_NAME{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 reserved[2];
    UINT32 returnHandle;
    union{
	UINT32 oDataPtr;/* only for Get Adaptor Name */
	UINT32 reserved2;/* only for Set Adaptor Name */
    } PT;
    union{
	UINT8 reserved3[40];/* only for Get Adaptor Name */
	UINT8 name[40];/* only for Set Adaptor Name */
    } NM;
}STRUCT_MSG_IOCTL_ADAPTOR_NAME;

/**********************************************************
 *	Disk command message format:
 *		Disk DMA Timing
 *		Get Disk Name
 *		Disk Size
 *		Disk Status
 *		Disk Location
 */
typedef struct _STRUCT_MSG_DISK{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    union{
	UINT8 deviceId;
	UINT8 channelId;
    } ID;
    union{
	UINT8 reserved1;
	UINT8 index;
    } IX;
    UINT32 returnHandle;
    UINT32 oDataPtr;
    union{
	UINT32 reserved2;
	struct{
	    UINT8 reserved3[2];
	    UINT8 notUsed[2];
	} RD;
    } RD2;
}STRUCT_MSG_DISK;


/**********************************************************
 *	Array command message format:
 *		Array Description
 *		Array Status
 *		Array Number of Disk
 *		Array Delete
 *		Array Maintenance Stop
 *		Array Maintenance Lookup
 *		Array Integrity Check
 *		Array Rebuild Start
 *		Set Array Rebuild Rate
 */
typedef struct _STRUCT_MSG_ARRAY{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 arrayId;
    union{
	UINT8 deviceId; /* only for Array Rebuild Start */
	UINT8 reserved;
    } ID;
    UINT32 returnHandle;
    union{
	UINT32 oDataPtr;
	UINT32 rate;	/* only for Set Array Rebuild Rate */
    } PT;
    UINT32 reserved1;
}STRUCT_MSG_ARRAY;

/**********************************************************
 *	Array command message format:
 *		Array Create
 */
typedef struct _STRUCT_MSG_ARRAY_CREATE{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 reserved[2];
    UINT32 returnHandle;
    UINT32 oDataPrt;
    UINT32 raidMode;
    UINT32 stripBlockSize;
    UINT8 name[0x3C - 0x14];
    UINT8 reserved1;
    UINT8 notUsed[3];
    UINT8 numberOfDisk;
    UINT8 deviceId[REQ_MSG_SIZE_TYPE1 - 0x41];
}STRUCT_MSG_ARRAY_CREATE;

/**********************************************************
 *	Array command message format:
 *		Array Migrate Start
 */
typedef struct _STRUCT_MSG_ARRAY_MIGRATE{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 arrayId;
    UINT8 raidMode;
    UINT32 returnHandle;
    UINT8 numberOfDisk;
    UINT8 deviceId[REQ_MSG_SIZE_TYPE0-9];
}STRUCT_MSG_ARRAY_MIGRATE;

/**********************************************************
 *	Array command message format:
 *		Get Array Name
 *		Set Array Name
 */
typedef struct _STRUCT_MSG_ARRAY_NAME{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 arrayId;
    UINT8 reserved;
    UINT32 returnHandle;
    union{
	UINT32 oDataPtr;	/* only for Get Array Name */
	UINT32 reserved1;
    } PT;
    union{
	UINT8 reserved2[0x34 - 0x0C];
	UINT8 name[0x34 - 0x0C];/* only for Set Array Name */
    } NM;
}STRUCT_MSG_ARRAY_NAME;

/**********************************************************
 *	Cache command message format (Not supported now)
 *		Get Write Policy
 *		Set Write Policy
 *		Get Flush Frequency
 *		Set Flush Frequency
 */
typedef struct _STRUCT_MSG_CACHE_WRITE_POLICY{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 arrayId;
    UINT8 reserved;
    UINT32 returnHandle;
    union{
	UINT32 oDataPtr;	/* only used for Get Write Policy */
	UINT32 reserved2;
    } PT;
    union{
	UINT32 reserved3;
	UINT32 writePolicy;	/* only used for Set Write policy */
	struct{
	    UINT16 raidMode;
	    union{
		UINT16 notUsed;
		UINT16 frequency;
	    } FQ;
	} RM;
    } DT;
}STRUCT_MSG_CACHE;

/**********************************************************
 *	Miscellaneous command message format
 *		Allocate memory for Engine
 *		Get Carmel Time
 *		Set Carmel Time
 */
typedef struct _STRUCT_MSG_MISC1{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 reserved[2];
    UINT32 returnHandle;
    union{
	UINT32 oDataPtr;
	UINT32 reserved1;
	UINT32 startAddr;
    } PT;
    union{
	UINT32 reserved2;
	UINT32 timeStamp;
	UINT32 length;
    } DT;
}STRUCT_MSG_MISC_ALLOCATE_MEM, STRUCT_MSG_MISC_TIME;

typedef struct _STRUCT_MSG_MISC2{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 sgCount;
    UINT8 sgType;
    UINT32 returnHandle;
    UINT32 baseAddr;
    UINT32 length;
    struct _SG{
	UINT32 addr;
	UINT32 length;
    } sg[MAX_SG_COUNT];
    struct{
	UINT32 highAddr;
	UINT32 reserved;
    }SUPPORT_64BITS_ADDR;
}STRUCT_MSG_MISC_FW;

typedef struct _STRUCT_MSG_MISC3{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 reserved[2];
    UINT32 returnHandle;
    UINT8 channelCount;
    UINT8 channelId[REQ_MSG_SIZE_TYPE0 - 0x09];
}STRUCT_MSG_MISC_CHANNEL;

typedef struct _STRUCT_MSG_MISC4{
    union{
	UINT16 cmd;
	struct{
	    UINT8 type;
	    UINT8 subCmd;
	} TP;
    } HD;
    UINT8 reserved[2];
    UINT32 returnHandle;
    UINT32 oDataPtr;
    UINT32 reserved1;
}STRUCT_MSG_MISC_GET_FW_VERSION;

/**********************************************************
 *	Pass through command message format
 */
typedef struct _STRUCT_MSG_PASS_THROUGH{
    union{
	UINT8 cmd;
	UINT8 type;
	UINT8 subCmd;
    } HD;
    UINT8 channelId;
    UINT8 index;
    UINT8 sgCount;
    UINT32 returnHandle;
    union{
	UINT32 iDataPtr;
	UINT32 oDataPtr;
    } DP;
    UINT32 bufferLength;
    UINT32 sgTableAddr;
    UINT8 cmdCode;	/* ATA command code or ATAPI operation code */
    UINT8 cmdLength;	/* ATA/ATAPI command packet length */
    UINT8 paraLength;	/* ATA/ATAPI command's parameters' length */
    UINT8 sgType;
    UINT8 cmdPacket[REQ_MSG_SIZE_TYPE0-0x18];
}STRUCT_MSG_PASS_THROUGH;


/********************************************************
 *		Output Data format			*
 ********************************************************/





/********************************************************
 *		Event log format			*
 ********************************************************/
typedef struct _STRUCT_EVENT_LOG{
    UINT16 eventId;
    UINT8 eventType;
    UINT8 eventCode;
    UINT32 eventInfo;
    UINT32 timeStamp;
}STRUCT_EVENT_LOG;

#endif	/*ifndef _TYPE_H */



